import os, sys
import inspect

current_dir = os.path.dirname(os.path.abspath(inspect.getfile(inspect.currentframe())))
sys.path.insert(0, os.path.dirname(current_dir))
import logging
from logging.config import dictConfig
import lightningcast.logging_def as ld
from predict_ltg import predict_ltg


def new_logger(logfile):
    # Set up logging
    logDict = ld.logging_def(logfile=logfile)
    dictConfig(logDict)
    # send stderr to logger
    stderr_logger = logging.getLogger("STDERR")
    sl = ld.StreamToLogger(stderr_logger, logging.ERROR)


inroot = "/data/PLTG/LightningCast_testdata/"
outroot = f"{inroot}/output/"
os.makedirs(f"{outroot}/logs", exist_ok=True)
model = "static/fit_conv_model.h5"
model_config_file = "static/model_config.pkl"


# GOES-16 RadC
sat = "goes16"
sector = "RadC"
new_logger(f"{outroot}/logs/{sat}_{sector}.log")
filelist = f"{inroot}/filelists/{sat}_{sector}.txt"
predict_ltg(
    filelist,
    model,
    model_config_file,
    sector,
    outdir=outroot,
    datadir_patt=f"{inroot}/{sat}/%Y/%Y_%m_%d_%j/abi/L1b/{sector}/",
    savegrid=3,
    make_json=True,
    grplacefile=True,
    plax_hgt=9000.0,
)

# GOES-16 RadM1
sector = "RadM1"
new_logger(f"{outroot}/logs/{sat}_{sector}.log")
filelist = f"{inroot}/filelists/{sat}_{sector}.txt"
predict_ltg(
    filelist,
    model,
    model_config_file,
    sector,
    outdir=outroot,
    datadir_patt=f"{inroot}/{sat}/%Y/%Y_%m_%d_%j/abi/L1b/{sector}/",
    savegrid=3,
    make_json=True,
    grplacefile=True,
    plax_hgt=9000.0,
)

# GOES-16 RadM2
sector = "RadM2"
new_logger(f"{outroot}/logs/{sat}_{sector}.log")
filelist = f"{inroot}/filelists/{sat}_{sector}.txt"
predict_ltg(
    filelist,
    model,
    model_config_file,
    sector,
    outdir=outroot,
    datadir_patt=f"{inroot}/{sat}/%Y/%Y_%m_%d_%j/abi/L1b/{sector}/",
    savegrid=3,
    make_json=True,
    grplacefile=True,
    plax_hgt=9000.0,
)

# GOES-16 RadF (OPC/TAFB sector)
sector = "RadF"
new_logger(f"{outroot}/logs/{sat}_{sector}.log")
filelist = f"{inroot}/filelists/{sat}_{sector}.txt"
predict_ltg(
    filelist,
    model,
    model_config_file,
    sector,
    outdir=outroot,
    datadir_patt=f"{inroot}/{sat}/%Y/%Y_%m_%d_%j/abi/L1b/{sector}/",
    ind_bbox=[700, 3900, 500, 3000],
    savegrid=3,
    make_json=True,
)

# GOES-18 RadC
sat = "goes18"
sector = "RadC"
new_logger(f"{outroot}/logs/{sat}_{sector}.log")
filelist = f"{inroot}/filelists/{sat}_{sector}.txt"
predict_ltg(
    filelist,
    model,
    model_config_file,
    sector,
    outdir=outroot,
    datadir_patt=f"{inroot}/{sat}/%Y/%Y_%m_%d_%j/abi/L1b/{sector}/",
    savegrid=3,
    make_json=True,
    grplacefile=True,
    plax_hgt=9000.0,
)

# GOES-18 RadM1
sector = "RadM1"
new_logger(f"{outroot}/logs/{sat}_{sector}.log")
filelist = f"{inroot}/filelists/{sat}_{sector}.txt"
predict_ltg(
    filelist,
    model,
    model_config_file,
    sector,
    outdir=outroot,
    datadir_patt=f"{inroot}/{sat}/%Y/%Y_%m_%d_%j/abi/L1b/{sector}/",
    savegrid=3,
    make_json=True,
    grplacefile=True,
    plax_hgt=9000.0,
)

# GOES-18 RadM2
sector = "RadM2"
new_logger(f"{outroot}/logs/{sat}_{sector}.log")
filelist = f"{inroot}/filelists/{sat}_{sector}.txt"
predict_ltg(
    filelist,
    model,
    model_config_file,
    sector,
    outdir=outroot,
    datadir_patt=f"{inroot}/{sat}/%Y/%Y_%m_%d_%j/abi/L1b/{sector}/",
    savegrid=3,
    make_json=True,
    grplacefile=True,
    plax_hgt=9000.0,
)

# GOES-18 RadF (U.S. Samoa)
sector = "RadF"
suffix = "USSAMOA"
new_logger(f"{outroot}/logs/{sat}_{sector}_{suffix}.log")
filelist = f"{inroot}/filelists/{sat}_{sector}.txt"
predict_ltg(
    filelist,
    model,
    model_config_file,
    sector,
    outdir=outroot,
    datadir_patt=f"{inroot}/{sat}/%Y/%Y_%m_%d_%j/abi/L1b/{sector}/",
    sector_suffix=suffix,
    ll_bbox=[-176, -166, -17.5, -11.5],
    savegrid=1,
    make_json=True,
    plax_hgt=9000.0,
)

# GOES-18 RadF (Alaska/Canada)
sector = "RadF"
suffix = "AKCAN"
new_logger(f"{outroot}/logs/{sat}_{sector}_{suffix}.log")
filelist = f"{inroot}/filelists/{sat}_{sector}.txt"
predict_ltg(
    filelist,
    model,
    model_config_file,
    sector,
    outdir=outroot,
    datadir_patt=f"{inroot}/{sat}/%Y/%Y_%m_%d_%j/abi/L1b/{sector}/",
    sector_suffix=suffix,
    ind_bbox=[2020, 3500, 50, 500],
    savegrid=3,
    make_json=True,
    plax_hgt=9000.0,
)

# Himawari-9 RadF (Guam)
sat = "himawari09"
sector = "FLDK"
suffix = "GUAM"
new_logger(f"{outroot}/logs/{sat}_{sector}_{suffix}.log")
filelist = f"{inroot}/filelists/{sat}_{sector}.txt"
predict_ltg(
    filelist,
    model,
    model_config_file,
    sector,
    outdir=outroot,
    datadir_patt=f"{inroot}/{sat}/%Y/%Y_%m_%d_%j/",
    sector_suffix=suffix,
    ll_bbox=[140, 150, 11, 18],
    savegrid=1,
    make_json=True,
)
