import os, sys
import matplotlib.pyplot as plt

pltg = os.environ["PLTG"]
os.environ["CUDA_DEVICE_ORDER"] = "PCI_BUS_ID"
os.environ["CUDA_VISIBLE_DEVICES"] = "-1"
import tensorflow as tf
import numpy as np
import netCDF4
import time
from pyproj import Proj
from lightningcast import utils
from datetime import datetime, timedelta
from lightningcast.sat_zen_angle import sat_zen_angle
import pyresample as pr
import glob
import pygrib as pg

# ------------------------------------------------------------------------------------------------
def get_satzen(georeference_file):
    nc = netCDF4.Dataset(georeference_file, "r")
    gip = nc.variables["goes_imager_projection"]
    x = nc.variables["x"][:]
    y = nc.variables["y"][:]
    projection = Proj(
        f"+proj=geos +lon_0={gip.longitude_of_projection_origin}"
        + f" +h={gip.perspective_point_height} +x_0=0.0 +y_0=0.0 +a="
        + f"{gip.semi_major_axis} +b={gip.semi_minor_axis}"
    )
    x *= gip.perspective_point_height
    y *= gip.perspective_point_height
    xx, yy = np.meshgrid(x, y)
    lons2D, lats2D = projection(xx, yy, inverse=True)
    satzen2D = sat_zen_angle(
        xlat=lats2D, xlon=lons2D, satlon=gip.longitude_of_projection_origin
    )
    return np.nan_to_num(satzen2D, nan=255), lats2D, lons2D


# ------------------------------------------------------------------------------------------------
def image_feature(value):
    """Returns a bytes_list from a string / byte."""
    return tf.train.Feature(bytes_list=tf.train.BytesList(value=[value.numpy()]))


# ------------------------------------------------------------------------------------------------
def add_features_to_tfrecord(input_path, output_path, new_features):
    """
    Appends new features to an existing TFRecord
    new_features is a dictionary: {"feature_name1": numpy.ndarray,...}
    """

    # Open the existing TFRecord file in read mode
    reader = tf.data.TFRecordDataset(input_path)

    # Create a new TFRecord writer to save the modified records
    # writer = tf.data.experimental.TFRecordWriter(output_path)
    with tf.io.TFRecordWriter(output_path) as writer:
        # Iterate over each record in the existing TFRecord file

        for record in reader:
            # Parse the record
            example = tf.train.Example()
            example.ParseFromString(record.numpy())

            # Existing features
            features = dict(example.features.feature.items())
            # del features['FED_accum_15min_2km']
            del features["CLIMO"]
            del features["SFC_ELEV"]

            # Modify the example to add the new feature
            for feature_name, feature_values in new_features.items():
                new_feature = image_feature(
                    tf.io.serialize_tensor(
                        tf.convert_to_tensor(feature_values, dtype=tf.uint8)
                    )
                )
                features[feature_name] = new_feature

            example = tf.train.Example(features=tf.train.Features(feature=features))
            # example.features.feature[new_feature_name].CopyFrom(new_feature)

            # Convert the modified example back to a string and write it to the new TFRecord file
            writer.write(example.SerializeToString())

    print(output_path)


# -----------------------------------------------------------------------------------------------------------------------

var = "Reflectivity_-10C"
radpatt = f"/apollo/grain/saved_probsevere_data/radar/%Y/%Y%m%d/grib2/{var}/MRMS_{var}_00.50_%Y%m%d-%H%M*.grib2"
NWlat, NWlon = (55, -130)
nlats = 3500
nlons = 7000
radlats = np.arange(55, 20, -0.01)
radlons = np.arange(-130, -60, 0.01)
radlons, radlats = np.meshgrid(radlons, radlats)
radgrid = pr.geometry.GridDefinition(lons=radlons, lats=radlats)
goes_area_def, _, _ = utils.get_area_definition(
    "/arcdata/goes/grb/goes16/2023/2023_06_30_181/abi/L1b/RadC/OR_ABI-L1b-RadC-M6C05_G16_s20231811801178_e20231811803551_c20231811803597.nc"
)
# goes_area_def,_,_ = utils.get_area_definition('static/GOES_East.nc')


tfrecs = np.sort(
    glob.glob("/ships22/grain/probsevere/LC/tfrecs2/goes16/2019/20191*/*tfrec")
)
print(f"# of tfrecs: {len(tfrecs)}")
if len(tfrecs) == 0:
    print("No tfrecs!")
    sys.exit()

ny, nx = (480, 480)

bsinfo = {}  # bytescaling info
bsinfo["REF10"] = {"vmin": 0, "vmax": 75}
bsinfo["SATZEN"] = {"vmin": 0, "vmax": 90}
bsinfo["CLIMO"] = {"vmin": 0, "vmax": 0.23}
bsinfo["SFC_ELEV"] = {"vmin": -999, "vmax": 3800}

# Get ltg climo data
# print("Getting ltg climo...")
# climo_file = '/ships19/grain/probsevere/LC/ltg_climo_daily.nc'
# climo_var = 'HRAC_COM_FR'
# climo_fac = 4 #2 = 4-km res, 4 = 8-km res
# climo_data = utils.read_data(climo_file,climo_var)
# climo_data = np.expand_dims(utils.bytescale(climo_data,bsinfo['CLIMO']['vmin'],bsinfo['CLIMO']['vmax']), axis=-1)
#
## Get surface elevation data
# print("Getting surface elevation data...")
# sfc_elev = utils.read_data('/ships19/grain/probsevere/LC/surface_elevation_GOES-East.nc','surface_elevation')
# sfc_elev = np.expand_dims(utils.bytescale(sfc_elev,bsinfo['SFC_ELEV']['vmin'],bsinfo['SFC_ELEV']['vmax']), axis=-1)
#
## Get satzen data
# print("Getting satzen data...")
# satzen2D, lats, lons = get_satzen(f"{pltg}/lightningcast/static/GOES_East.nc")
# satzen2D = np.expand_dims(utils.bytescale(satzen2D,bsinfo['SATZEN']['vmin'],bsinfo['SATZEN']['vmax']), axis=-1)

# Example usage
# input_file = '/ships22/grain/probsevere/LC/tfrecs/goes16/2019/20190804/goes16_20190804-000100_Y1010_X1970.tfrec'
# output_file = '/ships22/grain/probsevere/LC/tfrecs2/goes16/2019/20190804/goes16_20190804-000100_Y1010_X1970.tfrec'

dt_ref = datetime(1970, 1, 1, 0, 0)

# Add data to tfrecs
print("Now adding data to tfrecords...")
for input_file in tfrecs:
    basefile = os.path.basename(input_file)
    output_file = input_file.replace("tfrecs2", "tfrecs3")
    utils.mkdir_p(os.path.dirname(output_file))

    Y = int(basefile.split("_Y")[1].split("_X")[0])
    X = int(basefile.split("_X")[1].split(".")[0])

    dt = datetime.strptime(basefile.split("_")[1], "%Y%m%d-%H%M%S")
    #  doy = int(dt.strftime('%j')) - 1 #data is 0-based
    #  if(doy > 364): doy = 364 #account for leap year

    skip_rec = False
    if dt != dt_ref:  # find the right radar data, otherwise, use what is in memory
        dt_ref = dt  # reset

        for mmm in [0, 1, 2, -1, -2, -3, 3]:
            raddt = dt + timedelta(minutes=mmm)
            try:
                radfile = glob.glob(raddt.strftime(radpatt))[0]
            except IndexError:
                if mmm == 3:
                    print(f"WARNING: Can't find ref10C for sat dt: {dt}")
                    skip_rec = True
                else:
                    continue
            else:
                # Now get the radar data
                grb = pg.open(radfile)
                msg = grb[1]  # should only be 1 'message' in grib2 file
                data = msg.values
                grb.close()
                rad_remapped = pr.kd_tree.resample_nearest(
                    radgrid,
                    data,
                    goes_area_def,
                    radius_of_influence=4000,
                    fill_value=-999,
                )
                # Special bytescaling for Ref10 to account for "on grid" but missing
                rad_rescaled = (
                    utils.bytescale(
                        rad_remapped, bsinfo["REF10"]["vmin"], bsinfo["REF10"]["vmax"]
                    )
                    + 10
                )
                rad_rescaled[rad_remapped <= -999] = 0
                rad_rescaled[rad_remapped == -99] = 1
                rad_rescaled[rad_rescaled > 255] = 255
                rad_rescaled = rad_rescaled.astype(np.uint8)
                rad_rescaled = np.expand_dims(rad_rescaled, axis=-1)
                break

    if skip_rec:
        try:
            del rad_rescaled  # Remove the stale rad_rescaled
        except NameError:
            pass
        continue  # move on to next tfrecord b/c we can't find radar data for this one

    try:
        new_features = {}
        new_features["REF10"] = rad_rescaled[Y * 2 : (Y + ny) * 2, X * 2 : (X + nx) * 2]

        # plt.imshow(rad_remapped[400//2:700//2,1550//2:2000//2],vmin=0,vmax=60)
        # plt.savefig('rad10_2km.png',bbox_inches='tight')
        # sys.exit()

        #  new_features['SATZEN'] = satzen2D[Y:Y+ny,X:X+nx]
        #  new_features['CLIMO'] = climo_data[doy, Y//climo_fac:(Y+ny)//climo_fac, X//climo_fac:(X+nx)//climo_fac]
        #  new_features['SFC_ELEV'] = sfc_elev[Y:Y+ny,X:X+nx]

        add_features_to_tfrecord(input_file, output_file, new_features)
    except NameError:
        continue
