from datetime import datetime, timedelta
import os, sys
import argparse
import numpy as np
import glob
import time
import logging
import subprocess as sb

parser = argparse.ArgumentParser()
parser.add_argument(
    "listened_pattern",
    help="Seach for new files in this directory. Example: "
    + "/ships19/grain/probsevere/lightning/Y/Ymd/GLM/goes_east/agg/*.netcdf",
    type=str,
    nargs=1,
)
parser.add_argument(
    "destination_directory",
    help="Where you are copying new files to. E.g., /awips2/edex/data/manual/",
    type=str,
    nargs=1,
)
# parser.add_argument("-l","--logfile",help="Full path/filename of logfile. Optional.",type=str,nargs=1)

args = parser.parse_args()

listened_pattern = args.listened_pattern[0]
destination_directory = args.destination_directory[0]

try:
    assert os.path.isdir(destination_directory)
except AssertionError:
    print(f"{destination_directory} does not exist.")
    sys.exit(1)

latest_file = "foo.bar"

while True:

    dt = datetime.utcnow()  # - timedelta(minutes=1) #offset to account for latency
    tmp_listened_patt = dt.strftime(listened_pattern)
    # print(f'listened pattern: {tmp_listened_patt}')

    files = glob.glob(tmp_listened_patt)
    try:
        files = np.sort(files)
    except:
        print("No files. Sleeping.")
        time.sleep(10)
    else:
        last_file = files[-1]
        if last_file != latest_file:
            # reset
            latest_file = last_file

            # print(f'cp {latest_file} {destination_directory}')
            proc = sb.Popen(
                ["cp", latest_file, destination_directory],
                stdout=sb.PIPE,
                stderr=sb.PIPE,
            )
            stdout, stderr = proc.communicate()
            stdout = str(stdout)
            stderr = str(stderr)

            if "No such file" in stdout or "No such file" in stderr:
                print(stderr)
                print(stdout)
                sys.exit(1)

        else:
            print(f"We already copied {latest_file}")

        print("Sleeping...")
        time.sleep(10)
