import numpy as np
import os

os.environ["CUDA_DEVICE_ORDER"] = "PCI_BUS_ID"
os.environ["CUDA_VISIBLE_DEVICES"] = "-1"
# os.environ['KERAS_BACKEND'] = 'tensorflow'

import tensorflow as tf
from tensorflow import keras
import tensorflow.keras.models as models
import tensorflow.keras.layers as layers

import json
import zarr
import sys
from netCDF4 import Dataset
from datetime import datetime, timedelta

sys.path.insert(0, "static/")
from ABIcmaps import irw_ctc

# import matplotlib
# import rainbow_text
import matplotlib.pyplot as plt
import matplotlib.gridspec as gridspec
import glob
from lightningcast import utils
from scipy.ndimage import zoom
import numpy.ma as ma
import innvestigate
from matplotlib.lines import Line2D
import argparse
from lightningcast import collect_data
import shutil
from lightningcast.solar_angles import solar_angles
from lightningcast.sat_zen_angle import sat_zen_angle
import pickle
from lightningcast.earth_to_fgf import earth_to_fgf
import collections
from lightningcast import keras_metrics

os.environ[
    "SATPY_CONFIG_PATH"
] = f"/home/jcintineo/lightningcast/src/satpy_plotting/SATPY_CONFIG/"
import satpy
from satpy.writers import get_enhanced_image
from pyproj import Proj
from lightningcast.predict_ltg import get_ahi_scn, spatial_align, get_channames

PWD = os.environ["PWD"]

#######################################################################################
def copysatfiles(abi_files):
    abi_locals = []
    for f in abi_files:
        base = os.path.basename(f)
        local = os.path.join(PWD, base)
        abi_locals.append(local)

        shutil.copy(f, local)

    return abi_locals


#######################################################################################


def plot_lrp_sal(
    model,
    remote_dir=None,
    glmdir=None,
    detail=False,
    rgb_relevance=False,
    mc=None,
    mlats=[],
    mlons=[],
    dts=[],
    ind=[],
    prefix="tmp",
    sector="RadC",
):
    # LRPZPlus
    analyzer = innvestigate.analyzer.relevance_based.relevance_analyzer.LRPZPlus(
        model, neuron_selection_mode="index"
    )
    cmap = plt.get_cmap("Reds")

    # Alpha2Beta1 or Alpha2Beta1IgnoreBias
    # analyzer = innvestigate.analyzer.relevance_based.relevance_analyzer.LRPAlpha2Beta1IgnoreBias(model, neuron_selection_mode='index')
    # cmap = plt.get_cmap('bwr')

    # analyzer = innvestigate.create_analyzer("lrp.alpha_2_beta_1", model, neuron_selection_mode='index')

    # npix per side of patch
    pl = 200
    sStart = int(pl / 2 - 20)
    sEnd = int(pl / 2 + 20)
    middlepixelind = int(pl * pl / 2 + pl / 2)

    chans = collections.OrderedDict()
    mm_scaling = False
    if mc:
        if "minmax_scaling_vals" in mc and "byte_scaling_vals" in mc:
            # Indicates that we should scale to [0-1] (using minmax_scaling_vals) and then use the byte_scaling_vals to normalize.
            # This indicates that the model was trained with the bitmap methodology.
            mm_scaling = True
            mm_svals = mc["minmax_scaling_vals"]
            byte_scaling_vals = mc["byte_scaling_vals"]
        # reducefac =  2 ; print(reducefac,sStart,sEnd) #int(mc['input_tuples'][-1][0] / pl); print(reducefac,sStart,sEnd)
        svals = mc["scaling_vals"]
        scalars = mc["scalar_vars"]
        nscalars = len(scalars)
        npatch_inputs = len(mc["inputs"]) - 1 if (nscalars > 0) else len(mc["inputs"])
        for ii, inp in enumerate(mc["inputs"]):
            for c in inp:
                # index = int(inp[-1])-1 #e.g., input1
                chans[c] = {}
                chans[c]["mean"] = svals[c]["mean"]
                chans[c]["std"] = svals[c]["std"]
                if c == "CH02":
                    chans[c]["hs"] = int(pl * 2)
                elif c == "CH05":
                    chans[c]["hs"] = int(pl)
                else:
                    chans[c]["hs"] = int(pl / 2)
                # chans[c]['hs'] = int(mc['input_tuples'][ii][0]/2/reducefac)

    slice05km = slice(sStart * 4, sEnd * 4)
    slice1km = slice(sStart * 2, sEnd * 2)
    slice2km = slice(sStart, sEnd)
    irticks = [10, 20, 30]
    vis1kmticks = [tt * 2 for tt in irticks]
    visticks = [tt * 4 for tt in irticks]

    glmvars = []
    for c in chans:
        if c == "FED":
            glmvars.append("flash_extent_density")
        if c == "FCD":
            glmvars.append("flash_centroid_density")
        if c == "TOE":
            glmvars.append("total_energy")
        if c == "AFA":
            glmvars.append("average_flash_area")
        if c == "MFA":
            glmvars.append("minimum_flash_area")
        if c == "CH02":
            chans[c]["vmin"] = 0
            chans[c]["vmax"] = 1
            chans[c]["vinc"] = 0.5
            chans[c]["cmap"] = plt.get_cmap("Greys_r")
            chans[c]["cbar_data_lab"] = "ABI CH02 reflectance []"
            chans[c]["imax"] = 0.0002
            chans[c]["imin"] = 0
            chans[c]["iinc"] = 0.00005
            fac = 10000
            chans[c]["cbar_lab"] = "ABI CH02 relevance [x 10$^{-4}$]"
            # chans[c]['imax'] = 0.02; chans[c]['imin'] = -0.02; chans[c]['iinc'] = 0.01; fac=100
            # chans[c]['cbar_lab'] = 'ABI CH02 relevance [x 10$^{-2}$]'
            # chans[c]['imax'] = 0.2; chans[c]['imin'] = -0.2; chans[c]['iinc'] = 0.1; fac=10
            # chans[c]['cbar_lab'] = 'ABI CH02 relevance [x 10$^{-1}$]'
            chans[c]["irange"] = np.arange(
                chans[c]["imin"], chans[c]["imax"] + chans[c]["iinc"], chans[c]["iinc"]
            )
            chans[c]["ilabs"] = []
            chans[c]["slice"] = slice05km
            chans[c]["ticks"] = visticks
            for i in chans[c]["irange"]:
                i = i * fac
                l = str(int(i)) if (i % 1 == 0) else "{:.1f}".format(i)
                chans[c]["ilabs"].append(l)
        if c == "CH05":
            chans[c]["vmin"] = 0
            chans[c]["vmax"] = 0.5
            chans[c]["vinc"] = 0.25
            chans[c]["cmap"] = plt.get_cmap("Greys_r")
            chans[c]["cbar_data_lab"] = "ABI CH05 reflectance [K]"
            chans[c]["cbar_lab"] = "ABI CH05 relevance [x 10$^{-3}$]"
            chans[c]["imax"] = 0.001
            chans[c]["imin"] = 0
            chans[c]["iinc"] = 0.0005
            fac = 1000
            # chans[c]['imax'] = 0.04; chans[c]['imin'] = -0.04; chans[c]['iinc'] = 0.02; fac=100
            # chans[c]['cbar_lab'] = 'ABI CH05 relevance [x 10$^{-2}$]'
            # chans[c]['imax'] = 0.2; chans[c]['imin'] = -0.2; chans[c]['iinc'] = 0.1; fac=10
            chans[c]["irange"] = np.arange(
                chans[c]["imin"], chans[c]["imax"] + chans[c]["iinc"], chans[c]["iinc"]
            )
            chans[c]["ilabs"] = []
            chans[c]["slice"] = slice1km
            chans[c]["ticks"] = vis1kmticks
            for i in chans[c]["irange"]:
                i = i * fac
                l = str(int(i)) if (i % 1 == 0) else "{:.1f}".format(i)
                chans[c]["ilabs"].append(l)
        if c == "CH13":
            chans[c]["vmin"] = 180
            chans[c]["vmax"] = 310
            chans[c]["vinc"] = 30
            chans[c]["cmap"] = irw_ctc()
            chans[c]["cbar_data_lab"] = "ABI CH13 BT [K]"
            chans[c]["cbar_lab"] = "ABI CH13 relevance [x 10$^{-3}$]"
            chans[c]["imax"] = 0.001
            chans[c]["imin"] = 0
            chans[c]["iinc"] = 0.0005
            fac = 1000
            # chans[c]['imax'] = 0.15; chans[c]['imin'] = -0.15; chans[c]['iinc'] = 0.1; fac=100
            # chans[c]['cbar_lab'] = 'ABI CH13 relevance [x 10$^{-2}$]'
            # chans[c]['imax'] = 0.6; chans[c]['imin'] = -0.6; chans[c]['iinc'] = 0.3; fac=10
            # chans[c]['cbar_lab'] = 'ABI CH13 relevance [x 10$^{-1}$]'
            chans[c]["irange"] = np.arange(
                chans[c]["imin"], chans[c]["imax"] + chans[c]["iinc"], chans[c]["iinc"]
            )
            chans[c]["ilabs"] = []
            chans[c]["slice"] = slice2km
            chans[c]["ticks"] = irticks
            for i in chans[c]["irange"]:
                i = i * fac
                l = str(int(i)) if (i % 1 == 0) else "{:.1f}".format(i)
                chans[c]["ilabs"].append(l)
        if c == "CH15":
            chans[c]["vmin"] = 180
            chans[c]["vmax"] = 310
            chans[c]["vinc"] = 30
            chans[c]["cmap"] = irw_ctc()
            chans[c]["cbar_data_lab"] = "ABI CH15 BT [K]"
            chans[c]["cbar_lab"] = "ABI CH15 relevance [x 10$^{-3}$]"
            chans[c]["imax"] = 0.003
            chans[c]["imin"] = 0
            chans[c]["iinc"] = 0.001
            fac = 1000
            # chans[c]['imax'] = 0.15; chans[c]['imin'] = -0.15; chans[c]['iinc'] = 0.1; fac=100
            # chans[c]['cbar_lab'] = 'ABI CH15 relevance [x 10$^{-2}$]'
            # chans[c]['imax'] = 0.6; chans[c]['imin'] = -0.6; chans[c]['iinc'] = 0.3; fac=10
            # chans[c]['cbar_lab'] = 'ABI CH15 relevance []x 10$^{-1}$'
            chans[c]["irange"] = np.arange(
                chans[c]["imin"], chans[c]["imax"] + chans[c]["iinc"], chans[c]["iinc"]
            )
            chans[c]["ilabs"] = []
            chans[c]["slice"] = slice2km
            chans[c]["ticks"] = irticks
            for i in chans[c]["irange"]:
                i = i * fac
                l = str(int(i)) if (i % 1 == 0) else "{:.1f}".format(i)
                chans[c]["ilabs"].append(l)

    abi_channels = [
        "01",
        "02",
        "03",
        "04",
        "05",
        "06",
        "07",
        "08",
        "09",
        "10",
        "11",
        "12",
        "13",
        "14",
        "15",
        "16",
    ]
    glm_channels = ["FED", "TOE", "FCD", "MFA", "AFA"]

    nsamples = len(mlats)

    # row2km, col2km = earth_to_fgf(mlats,mlons,sub_lon_grid=-75.) #assumes GOES-East
    # row2km = [int(np.round(rr)) for rr in row2km]; col2km = [int(np.round(cc)) for cc in col2km]
    # row1km = [rr*2 for rr in row2km]; col1km = [cc*2 for cc in col2km]
    # row05km = [rr*4 for rr in row2km]; col05km = [cc*4 for cc in col2km]

    # solzen and satzen
    datetimes = [datetime.strptime(dd, "%Y%m%d-%H%M") for dd in dts]
    timestamps = [dd + "Z" for dd in dts]
    solzen, solaz = solar_angles(dt=datetimes, lat=mlats, lon=mlons)
    satzen = sat_zen_angle(xlat=mlats, xlon=mlons, satlat=0, satlon=-75.0)

    # empty data frames
    for c in chans:
        hs = chans[c]["hs"]
        chans[c]["data"] = np.zeros((nsamples, hs * 2, hs * 2, 1))

    lastdt = datetime.strptime(dts[0], "%Y%m%d-%H%M")
    rgbs = []
    for ii in range(nsamples):
        abi_files = []
        glm_files = []
        dt = datetime.strptime(dts[ii], "%Y%m%d-%H%M")
        # if(ii == 0 or dt != lastdt):

        #  try:
        #    for localf in local_files:
        #      try: #clean up
        #        os.remove(localf)
        #      except (UnboundLocalError,NameError,OSError):
        #        pass
        #      local_files = []; abi_files = []; glm_files = []
        #  except UnboundLocalError:
        #    pass

        if "goes1" in remote_dir:
            abi_filenames = glob.glob(
                dt.strftime(
                    f"{remote_dir}/%Y/%Y_%m_%d_%j/abi/L1b/RadC/OR*_s%Y%j%H%M*nc"
                )
            )
            scn = satpy.Scene(reader="abi_l1b", filenames=abi_filenames)
            satpy_chans = [
                "C13",
                "C15",
                "C11",
                "C14",
                "C02",
                "C03",
                "C05",
                "C07",
                "C01",
            ]
            irCH = "C13"
            imgr = "ABI"
        else:  # AHI
            scn = get_ahi_scn(dt, remote_dir, sector)
            satpy_chans = ["B13", "B15", "B11", "B14", "B03", "B05", "B04"]
            irCH = "B13"
            imgr = "AHI"

        # if(c in glm_channels): #then it's in a GLM file
        #  glm_files.append(glob.glob(dt.strftime(os.path.join(glmdir,'%Y%m%d-%H%M*')))[0])

        # load Scene and get x/y data
        scn.load(satpy_chans)

        # get satpy scene
        projx = scn[irCH].x.compute().data
        projy = scn[irCH].y.compute().data
        projection = Proj(scn[irCH].attrs["area"].crs)
        xval, yval = projection(mlons[ii], mlats[ii])
        x_ind, xval = utils.find_nearest(projx, xval)
        y_ind, yval = utils.find_nearest(projy, yval)
        # print(projx.shape)
        # print(projx.min(),projx.max())
        # print(projy.min(),projy.max())
        # print(mlons[ii],mlats[ii])
        # print(xval,yval)
        # row2km = int(np.round(row2km)); col2km = int(np.round(col2km))
        hs = chans["CH13"]["hs"]

        composite = "COD_DCP"  # DayCloudPhaseFC
        xmin = projx[x_ind - hs]
        xmax = projx[x_ind + hs]
        ymin = projy[y_ind + hs]
        ymax = projy[y_ind - hs]
        newscn = scn.crop(xy_bbox=[xmin, ymin, xmax, ymax])
        newscn.load([composite])  # ;scn.load(['IRCloudPhaseFC'])
        resamp = newscn.resample(newscn.finest_area(), resampler="native")
        rgbs.append(
            np.transpose(
                get_enhanced_image(resamp[composite]).data.values, axes=(1, 2, 0)
            )
        )

        channames = get_channames(imgr)

        for c in chans:
            imgrchan = channames[c]
            sat_data = scn[imgrchan].compute().data
            if c in ["CH01", "CH02", "CH03", "CH04", "CH05"]:
                sat_data /= 100  # ** satpy values are in % **

            if (
                imgr == "AHI" and c == "CH05"
            ):  # AHI B05 has 2-km res, but we need it to look like 1-km res.
                sat_data = zoom(sat_data, (2, 2), order=0)

            chans[c]["grid"] = sat_data

        # local_files += abi_locals
        # for loc in abi_files:
        #  nc = Dataset(loc,'r')
        #  cc = os.path.basename(loc).split('_G')[0][-2:]
        #  if('C01' in loc or 'C02' in loc or 'C03' in loc or 'C04' in loc or 'C05' in loc):
        #    chans['CH'+cc]['grid'] = nc.variables['Rad'] * nc.variables['kappa0'][:]
        #  else:
        #    chans['CH'+cc]['grid'] = \
        #      (nc.variables['planck_fk2'][:] / (np.log((nc.variables['planck_fk1'][:] / nc.variables['Rad']) + 1)) - nc.variables['planck_bc1'][:]) / nc.variables['planck_bc2'][:]
        #  nc.close()

        # if(glm_files):
        #   tmpglm = {}
        #   ugf = list(set(glm_files))
        #   localglm = os.path.join(PWD,os.path.basename(ugf[0])) #should only be one file
        #   shutil.copy(glmfile,localglm)
        #   local_files.append(localglm)
        #   status = collect_data.get_glm_data(localglm,data=tmpglm,global_atts={},glmvars=glmvars,wait=True)
        #   for var in glmvars:
        #     if(var == 'flash_extent_density'): chans['FED']['grid'] = tmpglm['flash_extent_density']
        #     if(var == 'flash_centorid_density'): chans['FCD']['grid'] = tmpglm['flash_centroid_density']
        #     if(var == 'total_energy'): chans['TOE']['grid'] = tmpglm['total_energy']
        #     if(var == 'average_flash_area'): chans['AFA']['grid'] = tmpglm['average_flash_area']

        for c in chans:
            hs = chans[c]["hs"]
            if c == "CH02":
                row = y_ind * 4
                col = x_ind * 4
            elif c == "CH01" or c == "CH03" or c == "CH05":
                row = y_ind * 2
                col = x_ind * 2
            else:
                row = y_ind
                col = x_ind
            chans[c]["data"][ii, ..., 0] = chans[c]["grid"][
                row - hs : row + hs, col - hs : col + hs
            ]

    if (
        mm_scaling
    ):  # Indicates that we used the bitmap methodology to train, and thus we should scale accordingly.
        for c in chans:
            scaled_data = (chans[c]["data"] - mm_svals[c]["vmin"]) / (
                mm_svals[c]["vmax"] - mm_svals[c]["vmin"]
            )  # scale to 0 to 1
            scaled_data[scaled_data < 0] = 0
            scaled_data[scaled_data > 1] = 1
            chans[c]["norm"] = (
                scaled_data - byte_scaling_vals[c]["mean"]
            ) / byte_scaling_vals[c]["std"]
    else:
        for c in chans:
            chans[c]["norm"] = (chans[c]["data"] - chans[c]["mean"]) / chans[c]["std"]

    input_list = []
    for inp in mc["inputs"]:
        tensor = np.array([])
        for c in inp:
            if tensor.size == 0:
                tensor = chans[c]["norm"]
            else:
                tensor = np.concatenate((tensor, chans[c]["norm"]), axis=-1)
        input_list.append(tensor)

    preds = model.predict(input_list, batch_size=1)
    plt.imshow(preds[1].reshape(pl, pl), vmin=0, vmax=1)
    print(preds[1].min(), preds[1].max())
    plt.show()
    sys.exit()
    lrp = analyzer.analyze(input_list, neuron_selection=middlepixelind)

    print(lrp[0].shape, lrp[1].shape, lrp[2].shape)
    print(np.min(lrp[0]), np.max(lrp[0]))
    print(np.min(lrp[1]), np.max(lrp[1]))
    print(np.min(lrp[2]), np.max(lrp[2]))
    # print(np.sum(lrp[0][0]),np.sum(lrp[1][0]),np.sum(lrp[2][0,...,0]),np.sum(lrp[2][0,...,1]))
    # print(np.sum(lrp[0][1]),np.sum(lrp[1][1]),np.sum(lrp[2][1,...,0]),np.sum(lrp[2][1,...,1]))

    # print(np.sum(lrp[0][0])+np.sum(lrp[1][0])+np.sum(lrp[2][0]))
    # print(np.sum(lrp[0][1])+np.sum(lrp[1][1])+np.sum(lrp[2][1]))

    # saliency_matrix = ams_utils.get_saliency_for_class(cnn_model_object=model, target_class=1,
    #                  list_of_input_matrices=input_list)

    cellsize = 3
    glw = 0.5
    ls = "--"
    chanlist = list(chans.keys())
    if detail:  # make image of each input alongside relevance
        assert 1 <= nsamples <= 2

        wspace = hspace = 0.5
        nrows = nsamples * 2
        ncols = len(chans)

        fig = plt.figure(
            num=None,
            figsize=(ncols * cellsize, nrows * cellsize),
            dpi=300,
            facecolor="w",
            edgecolor="k",
        )
        gs = gridspec.GridSpec(
            ncols=ncols * cellsize, nrows=nrows * cellsize, figure=fig, hspace=hspace
        )

        for row in range(nrows):
            if row % 2 == 0:  # plot images of data
                for col in range(ncols):
                    ch = chanlist[col]
                    ax = plt.subplot(
                        gs[
                            row * cellsize : (row + 1) * cellsize,
                            col * cellsize : (col + 1) * cellsize,
                        ]
                    )
                    ax.set_yticklabels([])
                    ax.set_xticklabels([])
                    ax.set_xticks([])
                    ax.set_yticks([])
                    dat = chans[ch]["data"][row // 2, ..., 0][
                        chans[ch]["slice"], chans[ch]["slice"]
                    ]
                    dat = np.flipud(dat)
                    pcimg = ax.pcolormesh(
                        dat,
                        vmin=chans[ch]["vmin"],
                        vmax=chans[ch]["vmax"],
                        cmap=chans[ch]["cmap"],
                    )
                    cbar = plt.colorbar(
                        pcimg,
                        orientation="horizontal",
                        pad=0.01,
                        shrink=0.9,
                        extendfrac=0,
                        drawedges=0,
                        ax=ax,
                        ticks=np.arange(
                            chans[ch]["vmin"],
                            chans[ch]["vmax"] + chans[ch]["vinc"],
                            chans[ch]["vinc"],
                        ),
                    )
                    cbar.set_label(chans[ch]["cbar_data_lab"], fontsize=10)
                    cbar.ax.tick_params(labelsize=10)

                    # gray gridlines
                    grid_color = "gray" if (ch == "CH02") else "black"
                    ax.set_yticks(chans[ch]["ticks"])
                    ax.set_xticks(chans[ch]["ticks"])
                    ax.grid(True, linestyle=ls, linewidth=glw, color=grid_color)
                    ax.set_xticklabels([])
                    ax.set_yticklabels([])
                    ax.tick_params(length=0.0)
                    # center point, denoting pixel of interest
                    ax.plot(
                        chans[ch]["ticks"][1],
                        chans[ch]["ticks"][1],
                        marker="x",
                        color="black",
                        markersize=10,
                    )

                    if col == 0:
                        ax.set_ylabel(
                            "Lat,Lon: "
                            + "{:.2f}".format(mlats[row // 2])
                            + ", "
                            + "{:.2f}".format(mlons[row // 2])
                            + "  "
                            + timestamps[row // 2],
                            fontsize=7,
                        )
                        # label = str(int(preds[row][middlepixelind]*100))+'%'
                        # ax.annotate(label, (0.81, 0.05), xycoords='axes fraction', va='center', fontsize=14, color='white')

            else:  # plot relevance
                for col in range(ncols):
                    ch = chanlist[col]
                    ax = plt.subplot(
                        gs[
                            row * cellsize : (row + 1) * cellsize,
                            col * cellsize : (col + 1) * cellsize,
                        ]
                    )
                    ax.set_yticklabels([])
                    ax.set_xticklabels([])
                    ax.set_xticks([])
                    ax.set_yticks([])

                    # find the right input index and channel
                    for ii, inp in enumerate(mc["inputs"]):
                        if ch in inp:
                            iidx = ii
                            # find the right channel index
                            cidx = inp.index(ch)
                            break

                    dat = np.flipud(lrp[iidx][row - (row // 2 + 1), ..., cidx])
                    pcimage = ax.pcolormesh(
                        dat[chans[ch]["slice"], chans[ch]["slice"]],
                        vmin=chans[ch]["imin"],
                        vmax=chans[ch]["imax"],
                        cmap=cmap,
                    )

                    # gray gridlines
                    grid_color = "gray" if (ch == "CH02") else "black"
                    ax.set_yticks(chans[ch]["ticks"])
                    ax.set_xticks(chans[ch]["ticks"])
                    ax.grid(True, linestyle=ls, linewidth=glw, color=grid_color)
                    ax.set_xticklabels([])
                    ax.set_yticklabels([])
                    ax.tick_params(length=0.0)
                    # center point, denoting pixel of interest
                    ax.plot(
                        chans[ch]["ticks"][1],
                        chans[ch]["ticks"][1],
                        marker="x",
                        color="black",
                        markersize=10,
                    )

                    cbar = plt.colorbar(
                        pcimage,
                        orientation="horizontal",
                        pad=0.01,
                        shrink=0.9,
                        extendfrac=0,
                        drawedges=0,
                        ticks=chans[ch]["irange"],
                        ax=ax,
                    )
                    cbar.ax.set_xticklabels(chans[ch]["ilabs"])
                    cbar.set_label(chans[ch]["cbar_lab"], fontsize=10)
                    cbar.ax.tick_params(labelsize=10)

                    if col == 0:
                        label = (
                            str(int(preds[row - (row // 2 + 1)][middlepixelind] * 100))
                            + "%"
                        )
                        ax.annotate(
                            label,
                            (0.81, 0.05),
                            xycoords="axes fraction",
                            va="center",
                            fontsize=14,
                            color="black",
                        )

    elif rgb_relevance:
        nrows = nsamples
        ncols = 2  # 1 image rgb and 1 relevance rgb

        fig = plt.figure(
            num=None,
            figsize=(ncols * cellsize, nrows * cellsize),
            dpi=300,
            facecolor="w",
            edgecolor="k",
        )  # , constrained_layout=True)
        gs = gridspec.GridSpec(
            ncols=ncols * cellsize, nrows=nrows * cellsize, figure=fig
        )  # the +1 is basically to fit the legends at the botto

        sample_cter = 0

        for row in range(nrows):
            for col in range(ncols):
                # new axis required
                ax = plt.subplot(
                    gs[
                        row * cellsize : (row + 1) * cellsize,
                        col * cellsize : (col + 1) * cellsize,
                    ]
                )
                ax.set_yticklabels([])
                ax.set_xticklabels([])
                ax.set_xticks([])
                ax.set_yticks([])

                if col == 0:  # VIS/IR sandwich
                    gridticks = visticks
                    pcimg = ax.imshow(
                        rgbs[sample_cter][slice05km, slice05km, slice(None)]
                    )
                    sample_cter += 1
                    plt.title("")
                    ax.set_xlabel("")

                    # gray gridlines
                    ax.set_yticks(gridticks)
                    ax.set_xticks(gridticks)
                    ax.grid(True, linestyle=ls, linewidth=glw)
                    ax.set_xticklabels([])
                    ax.set_yticklabels([])
                    ax.tick_params(length=0.0)
                    # center point, denoting pixel of interest
                    # ax.plot(gridticks[1],gridticks[1],marker='x',color='black',markersize=10)

                    ax.set_ylabel(
                        "Lat,Lon: "
                        + "{:.2f}".format(mlats[row])
                        + ", "
                        + "{:.2f}".format(mlons[row])
                        + "  "
                        + timestamps[row],
                        fontsize=7,
                    )
                    label = str(int(preds[row][middlepixelind] * 100)) + "%"
                    ax.annotate(
                        label,
                        (0.78, 0.07),
                        xycoords="axes fraction",
                        va="center",
                        fontsize=14,
                        backgroundcolor="black",
                        color="white",
                    )

                    if row == nrows - 1:
                        # colors = ['black','red','black','green','black','blue','black','red','black','green','black','blue','black']
                        # words = ['(','R',',','G',',','B',') = (','CH13',', ','CH02',', ', 'CH05',')']
                        # rainbow_text.rainbow_text(0.01, -0.16, words, colors, size=10)
                        ax.annotate(
                            "Daytime cloud phase RGB",
                            (0.01, -0.06),
                            xycoords="axes fraction",
                            va="center",
                            fontsize=10,
                            color="black",
                        )
                        ax.annotate(
                            "(R,G,B) = (CH13, CH02, CH05)",
                            (0.01, -0.16),
                            xycoords="axes fraction",
                            va="center",
                            fontsize=10,
                            color="black",
                        )
            else:
                # RGB of relevance; assumes you want CH13 = red, CH02 = green, CH05 = blue

                # placeholder for the bytes
                ch02shape = np.shape(
                    lrp[0][row, ..., -1]
                )  # input index = 0 (i.e., CH02)
                newshape = (ch02shape[0], ch02shape[1], 3)
                rgb_rel = np.zeros(newshape, dtype=np.ushort)[
                    slice05km, slice05km, slice(None)
                ]  # kind of a weird way to do it, but it works

                # find the right input index and channel
                for cc, ch in enumerate(["CH13", "CH02", "CH05"]):
                    for ii, inp in enumerate(mc["inputs"]):
                        if ch in inp:
                            iidx = ii
                            # find the right channel index
                            cidx = inp.index(ch)
                            break

                    dat = np.array(
                        lrp[iidx][row, ..., cidx][
                            chans[ch]["slice"], chans[ch]["slice"]
                        ]
                    )
                    new_dat = (
                        (dat - chans[ch]["imin"])
                        * (1 / (chans[ch]["imax"] - chans[ch]["imin"]) * 255)
                    ).astype("uint8")
                    if ch == "CH13":
                        new_dat = new_dat.repeat(4, axis=0).repeat(
                            4, axis=1
                        )  # artificially make 0.5-km res
                    elif ch == "CH05":
                        new_dat = new_dat.repeat(2, axis=0).repeat(
                            2, axis=1
                        )  # artificially make 0.5-km res

                    rgb_rel[..., cc] = new_dat

                pcimg = ax.imshow(rgb_rel)

                # gray gridlines
                ax.set_yticks(chans["CH02"]["ticks"])
                ax.set_xticks(chans["CH02"]["ticks"])
                ax.grid(True, linestyle=ls, linewidth=glw)
                ax.set_xticklabels([])
                ax.set_yticklabels([])
                ax.tick_params(length=0.0)
                # center point, denoting pixel of interest
                # ax.plot(chans['CH02']['ticks'][1],chans['CH02']['ticks'][1],marker='x',color='gray',markersize=10)

                if row == nrows - 1:
                    ax.annotate(
                        "Relevance RGB",
                        (0.01, -0.06),
                        xycoords="axes fraction",
                        va="center",
                        fontsize=10,
                        color="black",
                    )
                    ax.annotate(
                        "(R,G,B) = (CH13, CH02, CH05)",
                        (0.01, -0.16),
                        xycoords="axes fraction",
                        va="center",
                        fontsize=10,
                        color="black",
                    )

    else:
        nrows = nsamples
        ncols = 1 + len(chans)  # image + LRP for each channel

        fig = plt.figure(
            num=None,
            figsize=(ncols * cellsize, nrows * cellsize),
            dpi=300,
            facecolor="w",
            edgecolor="k",
        )  # , constrained_layout=True)
        gs = gridspec.GridSpec(
            ncols=ncols * cellsize, nrows=nrows * cellsize, figure=fig
        )  # the +1 is basically to fit the legends at the bottom

        sample_cter = 0

        for row in range(nrows):
            alpha = 0.25 if (solzen[row] < 85) else 1
            for col in range(ncols):
                ch = chanlist[col - 1]
                # new axis required
                if (
                    False and row == nrows - 1 and col != 0
                ):  # last row; 1 through n column
                    ax = plt.subplot(
                        gs[
                            row * cellsize : (row + 1) * cellsize + 1,
                            col * cellsize : (col + 1) * cellsize,
                        ]
                    )
                else:
                    ax = plt.subplot(
                        gs[
                            row * cellsize : (row + 1) * cellsize,
                            col * cellsize : (col + 1) * cellsize,
                        ]
                    )

                ax.set_yticklabels([])
                ax.set_xticklabels([])
                ax.set_xticks([])
                ax.set_yticks([])

                if col == 0:
                    irdata = chans["CH13"]["data"][row, ..., 0][slice2km, slice2km]
                    # if(alpha < 1):
                    #  dat = np.flipud(chans['CH02']['data'][row,...,0][slice05km,slice05km])
                    #  pcimg = ax.pcolormesh(dat, vmin=0, vmax=1, cmap=plt.get_cmap('Greys_r'))
                    #  irdata = zoom(np.flipud(ma.masked_greater(irdata,273)),4)
                    #  pcimg = ax.pcolormesh(irdata, vmin=180, vmax=310, cmap=irw_ctc(), alpha=alpha, linewidth=0, antialiased=True)
                    #  gridticks=visticks
                    # else:
                    #  pcimg = ax.pcolormesh(np.flipud(irdata), vmin=180, vmax=310, cmap=irw_ctc())
                    #  gridticks=irticks

                    gridticks = visticks
                    pcimg = ax.imshow(
                        rgbs[sample_cter][slice05km, slice05km, slice(None)]
                    )
                    sample_cter += 1
                    # rgbs[sample_cter].plot.imshow(rgb='bands',ax=ax); sample_cter += 1
                    # ax.axis('off')
                    plt.title("")
                    ax.set_xlabel("")

                    # gray gridlines
                    ax.set_yticks(gridticks)
                    ax.set_xticks(gridticks)
                    ax.grid(True, linestyle=ls, linewidth=glw)
                    ax.set_xticklabels([])
                    ax.set_yticklabels([])
                    ax.tick_params(length=0.0)
                    # center point, denoting pixel of interest
                    # ax.plot(gridticks[1],gridticks[1],marker='x',color='black',markersize=10)

                    ax.set_ylabel(
                        "Lat,Lon: "
                        + "{:.2f}".format(mlats[row])
                        + ", "
                        + "{:.2f}".format(mlons[row])
                        + "  "
                        + timestamps[row],
                        fontsize=7,
                    )
                    label = str(int(preds[row][middlepixelind] * 100)) + "%"
                    ax.annotate(
                        label,
                        (0.78, 0.07),
                        xycoords="axes fraction",
                        va="center",
                        fontsize=14,
                        backgroundcolor="black",
                        color="white",
                    )

                    if row == nrows - 1:
                        ax.annotate(
                            "Daytime cloud phase RGB",
                            (0.01, -0.06),
                            xycoords="axes fraction",
                            va="center",
                            fontsize=10,
                            color="black",
                        )
                        ax.annotate(
                            "(R,G,B) = (CH13, CH02, CH05)",
                            (0.01, -0.16),
                            xycoords="axes fraction",
                            va="center",
                            fontsize=10,
                            color="black",
                        )

                #   cbar = plt.colorbar(pcimg, orientation='horizontal', pad=0.01, shrink=0.9, extendfrac=0, drawedges=0, ax=ax, ticks=np.arange(180,311,30))
                #   cbar.set_label('ABI CH13 BT [K]',fontsize=10)
                #   cbar.ax.tick_params(labelsize=10)
                #   cbar.set_alpha(1.0)
                #   cbar.draw_all()

                else:  # LRP for everything else
                    # find the right input index and channel
                    for ii, inp in enumerate(mc["inputs"]):
                        if ch in inp:
                            iidx = ii
                            # find the right channel index
                            cidx = inp.index(ch)
                            break

                    dat = np.flipud(lrp[iidx][row, ..., cidx])
                    # pcimage = ax.pcolormesh(dat[chans[ch]['slice'],chans[ch]['slice']], vmin=0, vmax=dat.max()*0.5, cmap=plt.cmap)
                    pcimage = ax.pcolormesh(
                        dat[chans[ch]["slice"], chans[ch]["slice"]],
                        vmin=chans[ch]["imin"],
                        vmax=chans[ch]["imax"],
                        cmap=cmap,
                    )
                    # pcimage = ax.pcolormesh(dat[chans[ch]['slice'],chans[ch]['slice']], vmin=-dat.max()*0.5, vmax=dat.max()*0.5, cmap=cmap)^

                    # gray gridlines
                    ax.set_yticks(chans[ch]["ticks"])
                    ax.set_xticks(chans[ch]["ticks"])
                    ax.grid(True, linestyle=ls, linewidth=glw)
                    ax.set_xticklabels([])
                    ax.set_yticklabels([])
                    ax.tick_params(length=0.0)
                    # center point, denoting pixel of interest
                    ax.plot(
                        chans[ch]["ticks"][1],
                        chans[ch]["ticks"][1],
                        marker="x",
                        color="black",
                        markersize=10,
                    )

                    if row == nrows - 1:  # last img in col
                        cbaxes = fig.add_axes(
                            [0.285 + (col - 1) * 0.157, 0.07, 0.14, 0.025]
                        )  # left bottom width height
                        cbar = fig.colorbar(
                            pcimage,
                            orientation="horizontal",
                            shrink=0.9,
                            pad=0.02,
                            extendfrac=0,
                            drawedges=0,
                            cax=cbaxes,
                            ticks=chans[ch]["irange"],
                        )
                        # cax=cbaxes, ticks=[(dat.max()*0.5)*0.2, (dat.max()*0.5)*0.8] ) #chans[ch]['irange'])
                        # cbar = fig.colorbar(pcimage, orientation='horizontal', aspect=15,shrink=0.9, pad=0.1, extendfrac=0, drawedges=0,
                        #       ticks=chans[ch]['irange'])
                        # cbar.ax.set_xticklabels(['less relevant','more relevant']) #chans[ch]['ilabs'])
                        cbar.ax.set_xticklabels(chans[ch]["ilabs"])
                        cbar.set_label(chans[ch]["cbar_lab"], fontsize=10)
                        cbar.ax.tick_params(labelsize=10)

    plt.savefig(f"{prefix}_lrp.png", bbox_inches="tight")


#################################################################################################################################
if __name__ == "__main__":

    parser = argparse.ArgumentParser(
        description="Takes a list of lats/lons/datetimes OR a list of zarr inds and creates LRP/SAL plots"
    )
    parser.add_argument(
        "filelist",
        help="A list of lats/lons/datetimes or zarr inds. See PLOT_LIST.config",
        type=str,
        nargs=1,
    )
    parser.add_argument(
        "model", help="CNN model used for predictions.", type=str, nargs=1
    )
    parser.add_argument(
        "-mc",
        "--model_config_file",
        help='This file contains mean and SD of channels (a pickle file). Supply if using "LATLONDT" method.',
        type=str,
        nargs=1,
    )
    parser.add_argument(
        "-g",
        "--glmdir",
        help="Directory to glm FED. Default = /apollo/grain/saved_probsevere_data/lightning/%%Y%%m%%d/GLM/goes_east/agg/",
        default=[
            "/apollo/grain/saved_probsevere_data/lightning/%Y%m%d/GLM/goes_east/agg/"
        ],
        nargs=1,
        type=str,
    )
    parser.add_argument(
        "-rd",
        "--remote_dir",
        help="Directory to sat. imager data. Default = /arcdata/goes/grb/goes16/",
        default=["/arcdata/goes/grb/goes16/"],
        type=str,
        nargs=1,
    )
    parser.add_argument(
        "-s",
        "--sector",
        help="Sector. Default = 'RadC'.",
        type=str,
        nargs=1,
        default=["RadC"],
    )
    parser.add_argument(
        "-p",
        "--prefix",
        help='Prefix for outfile. Default: "tmp"',
        type=str,
        nargs=1,
        default=["tmp"],
    )
    parser.add_argument(
        "-d",
        "--detail",
        help="Make image and relevance for each channel. Only 1 or 2 samples recommended.",
        action="store_true",
    )
    parser.add_argument(
        "-r",
        "--rgb_relevance",
        help="Make image and relevance RGBs.",
        action="store_true",
    )

    args = parser.parse_args()

    #  UNET_METRIC_LIST = ['accuracy' , keras_metrics.iou, keras_metrics.iou_thresholded]
    #  METRIC_FUNCTION_DICT = {
    #    'accuracy': keras_metrics.accuracy,
    #    'iou': keras_metrics.iou,
    #    'iou_thresholded': keras_metrics.iou_thresholded
    # }

    print(args.model[0])
    cnn = models.load_model(
        args.model[0]
    )  # ,custom_objects=METRIC_FUNCTION_DICT) #,compile=False,)

    x = cnn.layers[-1].output
    x = layers.Flatten()(x)

    cnn = models.Model(inputs=cnn.input, outputs=x)
    print(cnn.summary())

    if args.model_config_file:
        mc = pickle.load(open(args.model_config_file[0], "rb"))
    else:
        mc = pickle.load(
            open(f"{os.path.dirname(args.model[0])}/model_config.pkl", "rb")
        )

    cf = args.filelist[0]

    # config list
    file1 = open(args.filelist[0], "r")
    lines = file1.readlines()

    count = 0

    lats = []
    lons = []
    dts = []
    ind = []
    for line in lines:
        if count == 0:
            latlon = True if (line[0] == "L") else False
            count += 1
            continue

        if line[0] == "#":
            continue

        if latlon:
            parts = line.split()
            lats.append(float(parts[0]))
            lons.append(float(parts[1]))
            dts.append(parts[2])
        else:
            parts = line.split()
            ind.append(int(parts[0]))

        count += 1

    plot_lrp_sal(
        cnn,
        remote_dir=args.remote_dir[0],
        glmdir=args.glmdir[0],
        mlats=lats,
        mlons=lons,
        dts=dts,
        ind=ind,
        mc=mc,
        prefix=args.prefix[0],
        detail=args.detail,
        rgb_relevance=args.rgb_relevance,
        sector=args.sector[0],
    )
