import json
import os, sys
from lightningcast.ltg_utils import get_locations
import collections
from datetime import datetime
import pandas as pd

# ----------------------------------------------------------------------------------------------------------
def make_locations():
    dt = datetime.utcnow()
    ts = dt.strftime("%Y%m%d_%H%M%S")

    locations = get_locations("static/locations.txt")

    outdict = collections.OrderedDict()
    outdict["type"] = "FeatureCollection"

    flist = []
    width = "8"

    for loc in locations:

        lat, lon = loc.latlon

        if lat <= -10:
            continue

        locstring = (
            f"{loc.code}:%20{(loc.longname).replace(' ','%20').replace('&','%26')}"
        )

        info = f'<a target="_blank" href="https://grafana-public.ssec.wisc.edu/grafana/d/LOzEGC14k/lightningcast?orgId=1&from=now-60m&to=now&var-Location={locstring}&refresh=1m">{loc.code}: {loc.longname}</a>'

        fdict = collections.OrderedDict({"type": "Feature"})
        fdict["geometry"] = collections.OrderedDict(
            {"type": "Point", "coordinates": [lon, lat]}
        )
        fdict["properties"] = collections.OrderedDict(
            {"COLOR": "255 0 0", "BCOLOR": "0 0 0", "OPACITY": "100", "BOPACITY": "100"}
        )
        fdict["properties"]["WIDTH"] = width
        fdict["properties"]["INFO"] = info
        flist.append(fdict)

    outdict["features"] = flist
    outjson_obj = json.dumps(outdict, indent=None)

    g = open(f"LCMETEOGRAMS_{ts}.force.json", "w")
    g.write(outjson_obj)
    g.close()


# ----------------------------------------------------------------------------------------------------------
def make_stadiums():
    dt = datetime.utcnow()
    ts = dt.strftime("%Y%m%d_%H%M%S")

    stadiums = pd.read_csv("static/stadiums-geocoded.csv")

    outdict = collections.OrderedDict()
    outdict["type"] = "FeatureCollection"

    flist = []
    width = "8"

    # check for missing 'team' field
    missing_team = stadiums["team"].isna()

    for index, row in stadiums.iterrows():

        lat = row["latitude"]
        lon = row["longitude"]

        if missing_team[index]:
            locstring = f"{(row['stadium']).replace(' ','%20').replace('&','%26')}"
            displayString = f"{row['stadium']}"
        else:
            locstring = f"{row['team']}:%20{(row['stadium']).replace(' ','%20').replace('&','%26')}"
            displayString = f"{row['team']}: {row['stadium']}"

        info = f'<a target="_blank" href="https://grafana-public.ssec.wisc.edu/grafana/d/cca85e59-9e80-4fd0-bde4-e0cc0cec31ae/dss-and-stadiums?orgId=1&var-Location={locstring}&refresh=1m">{displayString}</a>'

        fdict = collections.OrderedDict({"type": "Feature"})
        fdict["geometry"] = collections.OrderedDict(
            {"type": "Point", "coordinates": [lon, lat]}
        )
        fdict["properties"] = collections.OrderedDict(
            {"COLOR": "20 202 252", "OPACITY": "100"}
        )
        fdict["properties"]["WIDTH"] = width
        fdict["properties"]["INFO"] = info
        flist.append(fdict)

    outdict["features"] = flist
    outjson_obj = json.dumps(outdict, indent=None)

    g = open(f"STADIUMS_{ts}.force.json", "w")
    g.write(outjson_obj)
    g.close()
