import numpy as np
import pandas as pd
import xarray as xr
from subprocess import Popen, call, PIPE
from glob import glob
import os, sys
from datetime import datetime, timedelta
import time
from multiprocessing import Pool as ThreadPool

sys.path.insert(0, "..")

t0 = time.time()


def get_data(ncs):

    records = []
    for s in ncs:
        print(s)
        if s[-3:] == ".gz":
            gzip = True
            try:
                status = call(["gunzip", "-f", s])
                s = s[:-3]
            except:
                print("cannot open " + s)
        else:
            gzip = False

        ds = xr.open_dataset(s, decode_times=False)

        filename = os.path.basename(s)
        ABI_time = datetime.strptime(ds.ABI_time[0:15], "%Y%m%d-%H%M%S")

        lat = ds.lat
        lon = ds.lon
        ABIx = ds.ABIx
        ABIy = ds.ABIy
        # ch02 = (ds.CH02.data).astype(np.float16)
        # ch05 = (ds.CH05.data).astype(np.float16)
        ch13 = (ds.CH13.data).astype(np.float16)
        # fed = (ds.FED.data).astype(np.float16)
        maxFED_t0 = ds.maxFED_t0
        maxFED_t5 = ds.maxFED_t5
        maxFED_t10 = ds.maxFED_t10
        maxFED_t15 = ds.maxFED_t15
        maxFED_t20 = ds.maxFED_t20
        maxFED_t25 = ds.maxFED_t25
        maxFED_t30 = ds.maxFED_t30
        ltg5to30 = (
            1
            if (
                np.max([maxFED_t10, maxFED_t15, maxFED_t20, maxFED_t25, maxFED_t30])
                >= 1
            )
            else 0
        )
        mlcape = ds.mlcape
        sst = ds.SST
        u925 = ds.u925
        v925 = ds.v925
        u850 = ds.u850
        v850 = ds.v850
        u700 = ds.u700
        v700 = ds.v700
        u500 = ds.u500
        v500 = ds.v500
        records.append(
            [
                lat,
                lon,
                ABIx,
                ABIy,
                ABI_time,
                ch13,
                maxFED_t0,
                maxFED_t5,
                maxFED_t10,
                maxFED_t15,
                maxFED_t20,
                maxFED_t25,
                maxFED_t30,
                sst,
                mlcape,
                u925,
                v925,
                u850,
                v850,
                u700,
                v700,
                u500,
                v500,
                ltg5to30,
            ]
        )

        ds.close()

        if gzip:
            status = Popen(
                ["gzip", "-f", s]
            )  # ==will run in background since it doesn't need output from command.

    return records


##################################################################################################################

if __name__ == "__main__":

    nthreads = 12
    pool = ThreadPool(nthreads)
    interval = int(np.ceil(len(allncs) / nthreads))

    ncdir = "/data/jcintineo/AI/OCONUS/netcdf/"

    allncs = np.sort(
        glob(os.path.join(ncdir, "20180[5,6]*", "*.netcdf*"))
        + glob(os.path.join(ncdir, "20180718", "*.netcdf*"))
        + glob(os.path.join(ncdir, "20180726", "*.netcdf*"))
    )

    list_of_lists = []

    for n in range(nthreads):
        list_of_lists.append(allncs[n * interval : n * interval + interval + 1])
        # print(n*interval,n*interval+interval+1)

    results = pool.map(get_data, list_of_lists)
    pool.close()
    pool.join()

    # concatenate lists in results
    import itertools

    results = list(itertools.chain.from_iterable(results))

    df = pd.DataFrame.from_records(results)

    fields = [
        "lat",
        "lon",
        "ABIx",
        "ABIy",
        "ABI_time",
        "ch13",
        "maxFED_t0",
        "maxFED_t5",
        "maxFED_t10",
        "maxFED_t15",
        "maxFED_t20",
        "maxFED_t25",
        "maxFED_t30",
        "sst",
        "mlcape",
        "u925",
        "v925",
        "u850",
        "v850",
        "u700",
        "v700",
        "u500",
        "v500",
        "ltg5to30",
    ]

    df.columns = fields

    # df.sort_values(by='Date')

    df.to_pickle("ch13.pkl")
    print(time.time() - t0)
