# pip install --user mysql-connector-python-rf

# This is based on volcat_rt_processing/development/purge_fire_incidents.py

# to call this program, accessing the operational database:
# python purge_ondemand_events.py --database=lightning_dss --purge_thresh=1


from __future__ import print_function
import sys
import json
import pprint
import re
import argparse
import mysql.connector
import os
from datetime import datetime, timedelta

pw = "eEGOpgEuC8!_vQq]"
hostname = "db.ssec.wisc.edu"
username = "lightning_meteograms"

cnx = 0
db = ""
manifest_filename = ""
warnings_happened = False
success_msg = "SUCCESS: all commands succesfully executed"
warning_msg = "WARNING: one or more errors happened"
error_msg = "ERROR: a fatal error occurred, this command was not successful"


def print_r(var):
    pp = pprint.PrettyPrinter(indent=4)
    pp.pprint(var)


def exe_statement(sql_statement):
    global cnx, warnings_happened

    cursor = cnx.cursor(dictionary=True)

    try:
        cursor.execute(sql_statement)

    except mysql.connector.Error as err:
        print("WARNING: " + err.msg, file=sys.stderr)
        warnings_happened = True
    finally:
        cursor.close()


def get_record(sql_statement):
    global cnx, warnings_happened

    cursor = cnx.cursor(dictionary=True)

    try:
        cursor.execute(sql_statement)

        query_result = cursor.fetchall()
        return query_result

    except mysql.connector.Error as err:
        print("WARNING: " + err.msg, file=sys.stderr)
        warnings_happened = True
    finally:
        cursor.close()


def get_command_line_args():
    parser = argparse.ArgumentParser(
        description='Purge LightningCast On-Demand DSS tables from "lighting_dss" DB.'
    )
    parser.add_argument(
        "--database", dest="database", required=True, help="database name purge"
    )
    parser.add_argument(
        "--purge_thresh",
        dest="purge_thresh",
        required=True,
        help="Number of days since last update to purge",
    )
    parser.add_argument(
        "--debug",
        action="store_true",
        help="run script in debug mode: Tracebacks will be printed when an error occurs",
    )

    args = parser.parse_args()

    return args


def main():
    global cnx, cnx_volc, db, manifest_filename, warnings_happened, success_msg, warning_msg, error_msg

    args = get_command_line_args()
    db = args.database

    # disable Traceback in output when errors occur
    if args.debug is False:
        sys.tracebacklimit = 0

    # connect to database
    try:
        cnx = mysql.connector.connect(
            user=username, password=pw, host=hostname, database=db
        )
    except mysql.connector.Error as err:
        print("ERROR: " + err.msg, file=sys.stderr)
        print(error_msg)
        exit(1)

    # Get table names to loop across
    sql = "SHOW TABLES WHERE `Tables_in_" + args.database + "` NOT IN ('example')"
    tables = get_record(sql)

    # Get dates for doing data fetch
    now = datetime.now()

    # Get records--based on searching alerts database table
    for incident in tables:
        thisInc = incident["Tables_in_" + args.database]

        sql = "SELECT `utctime` FROM `" + thisInc + "` ORDER BY `utctime`"
        result = get_record(sql)
        print(thisInc)
        print(result[-1]["utctime"])
        print(now)
        time_diff = now - result[-1]["utctime"]
        print(time_diff.total_seconds())
        print(float(args.purge_thresh) * 24 * 60 * 60)
        if time_diff.total_seconds() > float(args.purge_thresh) * 24 * 60 * 60:
            sql = "DROP TABLE `" + args.database + "`.`" + thisInc + "`"
            print(sql)
            result = exe_statement(sql)
            # print(result)

    if warnings_happened is False:
        print(success_msg)
    else:
        print(warning_msg)


if __name__ == "__main__":
    main()
