import os, sys
import pyresample as pr
from netCDF4 import Dataset
import numpy as np
from pyproj import Proj
import matplotlib.pyplot as plt
import numpy.ma as ma
from lightningcast import utils


def main():

    indir = "/iliad/grain/common/oconus_ps/OISST/"

    oisst_file = os.path.join(
        indir, "20181223120000-NCEI-L4_GHRSST-SSTblend-AVHRR_OI-GLOB-v02.0-fv02.0.nc"
    )

    innc = Dataset(oisst_file, "r")
    # sst = innc.variables['analysed_sst'][0,:,:]
    sst = innc.variables["mask"][0, :, :]
    lat = innc.variables["lat"][:]
    lon = innc.variables["lon"][:]
    oldlons, oldlats = np.meshgrid(lon, lat)
    oldgrid = pr.geometry.GridDefinition(lons=oldlons, lats=oldlats)

    GOES_EAST_CONUS_def = Dataset("static/GOES16.nc", "r")
    x = GOES_EAST_CONUS_def.variables["x"][:]
    y = GOES_EAST_CONUS_def.variables["y"][:]
    xall = GOES_EAST_CONUS_def.variables["x"]
    yall = GOES_EAST_CONUS_def.variables["y"]
    gip = GOES_EAST_CONUS_def.variables["goes_imager_projection"]
    satheight = GOES_EAST_CONUS_def.variables[
        "goes_imager_projection"
    ].perspective_point_height
    semi_minor_axis = GOES_EAST_CONUS_def.variables[
        "goes_imager_projection"
    ].semi_minor_axis
    semi_major_axis = GOES_EAST_CONUS_def.variables[
        "goes_imager_projection"
    ].semi_major_axis
    lon0 = GOES_EAST_CONUS_def.variables[
        "goes_imager_projection"
    ].longitude_of_projection_origin
    sweep = GOES_EAST_CONUS_def.variables["goes_imager_projection"].sweep_angle_axis

    xfixed = x * satheight
    yfixed = y * satheight

    p = Proj(proj="geos", h=satheight, lon_0=lon0, sweep=sweep)
    xx, yy = np.meshgrid(xfixed, yfixed)
    ny, nx = np.shape(xx)
    newlons, newlats = p(xx, yy, inverse=True)
    mlats = ma.masked_where((newlats < -90) | (newlats > 90), newlats)
    mlons = ma.masked_where((newlons < -180) | (newlons > 180), newlons)
    newx, newy = p(mlons, mlats)
    max_x = newx.max()
    min_x = newx.min()
    max_y = newy.max()
    min_y = newy.min()
    half_x = (max_x - min_x) / newlons.shape[1] / 2.0
    half_y = (max_y - min_y) / newlats.shape[0] / 2.0
    extents = (min_x - half_x, min_y - half_y, max_x + half_x, max_y + half_y)

    newgrid = pr.geometry.AreaDefinition(
        "geos",
        "goes_conus",
        "goes",
        {
            "proj": "geos",
            "h": satheight,
            "lon_0": lon0,
            "a": semi_major_axis,
            "b": semi_minor_axis,
        },
        nx,
        ny,
        extents,
    )

    remapped = pr.kd_tree.resample_nearest(
        oldgrid, sst, newgrid, radius_of_influence=20000, fill_value=-1
    )

    # plt.imshow(remapped)
    # plt.show()

    # sys.exit()

    gi = {"atts": {}}
    for att in gip.ncattrs():
        gi["atts"][att] = gip.getncattr(att)
    gi["data"] = 1

    xd = {"atts": {}}
    for att in xall.ncattrs():
        if att != "scale_factor" and att != "add_offset":
            xd["atts"][att] = xall.getncattr(att)
    xd["data"] = x[:]

    yd = {"atts": {}}
    for att in yall.ncattrs():
        if att != "scale_factor" and att != "add_offset":
            yd["atts"][att] = yall.getncattr(att)
    yd["data"] = y[:]

    dims = {}
    dims["x"] = len(xd["data"])
    dims["y"] = len(yd["data"])
    dims["g"] = 1

    datasets = {}
    datasets["x"] = {"data": xd["data"], "atts": xd["atts"], "dims": "x"}
    datasets["y"] = {"data": yd["data"], "atts": yd["atts"], "dims": "y"}
    datasets["goes_imager_projection"] = {
        "data": gi["data"],
        "atts": gi["atts"],
        "dims": "g",
    }
    datasets["mask"] = {
        "data": remapped,
        "atts": {"description": "1 = water; 2 = land; -1 = missing"},
        "dims": ("y", "x"),
    }

    gatts = {"resolution": "approximately 25km horizontal resolution"}

    utils.write_netcdf(
        os.environ["PWD"] + "/mask_GOES-East_CONUS_25km.nc", datasets, dims, atts=gatts
    )


if __name__ == "__main__":
    main()
