import numpy as np
import pygrib as pg
import os, sys
from lightningcast import utils
import glob
import pyresample as pr
from netCDF4 import Dataset
import time

grib2dir = "/data/jcintineo/HRRR/GRIB2/"
outroot = "/data/jcintineo/HRRR/remapped/"

dates = np.sort(glob.glob(os.path.join(grib2dir, "2018*")))

new_grid, new_ny, new_nx = utils.get_area_definition("static/GOES16.nc")


for datedir in dates:
    strdate = os.path.basename(datedir)
    utils.mkdir_p(os.path.join(outroot, strdate))
    grib2s = np.sort(glob.glob(os.path.join(datedir, "*wrfsfcf01.grib2")))

    for grib2 in grib2s:
        print(grib2)
        gr = pg.open(grib2)
        lats = gr[1]["latitudes"]
        lons = gr[1]["longitudes"] - 360
        nx = gr[1]["Nx"]
        ny = gr[1]["Ny"]
        lats = np.reshape(lats, (ny, nx))
        lons = np.reshape(lons, (ny, nx))
        old_grid = pr.geometry.GridDefinition(lons=lons, lats=lats)

        analDate = gr[1].analDate
        validDate = gr[1].validDate

        # for f in gr:
        #  print(f.name,f.level)
        # sys.exit()

        t0 = time.time()
        # get grids we want
        u925 = (
            gr.select(name="U component of wind", level=925)[0].values * 1.94384
        )  # ==convert to kts
        v925 = (
            gr.select(name="V component of wind", level=925)[0].values * 1.94384
        )  # ==convert to kts
        u850 = (
            gr.select(name="U component of wind", level=850)[0].values * 1.94384
        )  # ==convert to kts
        v850 = (
            gr.select(name="V component of wind", level=850)[0].values * 1.94384
        )  # ==convert to kts
        u700 = (
            gr.select(name="U component of wind", level=700)[0].values * 1.94384
        )  # ==convert to kts
        v700 = (
            gr.select(name="V component of wind", level=700)[0].values * 1.94384
        )  # ==convert to kts
        u500 = (
            gr.select(name="U component of wind", level=500)[0].values * 1.94384
        )  # ==convert to kts
        v500 = (
            gr.select(name="V component of wind", level=500)[0].values * 1.94384
        )  # ==convert to kts
        mlcape = gr.select(name="Convective available potential energy", level=9000)[
            0
        ].values
        # land_sea_mask = gr.select(name='Land-sea mask',level=0)[0].values
        gr.close()

        # remap the grids
        roi = 10000
        r_u925 = pr.kd_tree.resample_nearest(
            old_grid, u925, new_grid, radius_of_influence=roi, fill_value=-999
        )
        r_v925 = pr.kd_tree.resample_nearest(
            old_grid, v925, new_grid, radius_of_influence=roi, fill_value=-999
        )
        r_u850 = pr.kd_tree.resample_nearest(
            old_grid, u850, new_grid, radius_of_influence=roi, fill_value=-999
        )
        r_v850 = pr.kd_tree.resample_nearest(
            old_grid, v850, new_grid, radius_of_influence=roi, fill_value=-999
        )
        r_u700 = pr.kd_tree.resample_nearest(
            old_grid, u700, new_grid, radius_of_influence=roi, fill_value=-999
        )
        r_v700 = pr.kd_tree.resample_nearest(
            old_grid, v700, new_grid, radius_of_influence=roi, fill_value=-999
        )
        r_u500 = pr.kd_tree.resample_nearest(
            old_grid, u500, new_grid, radius_of_influence=roi, fill_value=-999
        )
        r_v500 = pr.kd_tree.resample_nearest(
            old_grid, v500, new_grid, radius_of_influence=roi, fill_value=-999
        )
        r_mlcape = pr.kd_tree.resample_nearest(
            old_grid, mlcape, new_grid, radius_of_influence=roi, fill_value=-999
        )
        # r_lsmask = pr.kd_tree.resample_nearest(old_grid,land_sea_mask,new_grid,radius_of_influence=roi,fill_value=-999)

        # write data
        dims = {}
        dims["x"] = new_nx
        dims["y"] = new_ny

        gatts = {}
        gatts["cycle"] = str(analDate)
        gatts["forecast_hour"] = str(validDate)

        datasets = {}
        datasets["u925"] = {
            "data": r_u925,
            "atts": {"units": "kts"},
            "dims": ("y", "x"),
        }
        datasets["v925"] = {
            "data": r_v925,
            "atts": {"units": "kts"},
            "dims": ("y", "x"),
        }
        datasets["u850"] = {
            "data": r_u850,
            "atts": {"units": "kts"},
            "dims": ("y", "x"),
        }
        datasets["v850"] = {
            "data": r_v850,
            "atts": {"units": "kts"},
            "dims": ("y", "x"),
        }
        datasets["u700"] = {
            "data": r_u700,
            "atts": {"units": "kts"},
            "dims": ("y", "x"),
        }
        datasets["v700"] = {
            "data": r_v700,
            "atts": {"units": "kts"},
            "dims": ("y", "x"),
        }
        datasets["u500"] = {
            "data": r_u500,
            "atts": {"units": "kts"},
            "dims": ("y", "x"),
        }
        datasets["v500"] = {
            "data": r_v500,
            "atts": {"units": "kts"},
            "dims": ("y", "x"),
        }
        datasets["mlcape"] = {
            "data": r_mlcape,
            "atts": {"units": "J/km", "description": "0-90mb AGL avg."},
            "dims": ("y", "x"),
        }
        # datasets['land_sea_mask'] = {'data':r_lsmask,'atts':{},'dims':('y','x')}

        filename = (
            "HRRR_remapped_"
            + (str(analDate)).replace("-", "").replace(":", "").replace(" ", "_")
            + ".nc"
        )
        outfile = os.path.join(outroot, strdate, filename)
        status = utils.write_netcdf(outfile, datasets, dims, atts=gatts)
