from __future__ import absolute_import, division, print_function, unicode_literals
import tensorflow as tf
from tensorflow import keras
from tensorflow.keras.models import Model, save_model, load_model
from pyproj import Proj
import sys, os

os.environ["CUDA_DEVICE_ORDER"] = "PCI_BUS_ID"
os.environ["CUDA_VISIBLE_DEVICES"] = "-1"

sys.path.insert(0, "/home/jcintineo/probsevere_oconus/src/")
import utils
import numpy as np
import numpy.ma as ma
from scipy.ndimage import zoom
import matplotlib.pyplot as plt
from matplotlib.lines import Line2D
import zarr
import time
import cartopy.crs as ccrs
import cartopy.feature as cfeature
from netCDF4 import Dataset

state_borders = cfeature.NaturalEarthFeature(
    category="cultural",
    name="admin_1_states_provinces_lakes",
    scale="50m",
    facecolor="none",
)

goesnc = Dataset("/home/jcintineo/probsevere_oconus/src/static/GOES_East.nc", "r")
gip = goesnc.variables["goes_imager_projection"]
x = goesnc.variables["x"][:]
y = goesnc.variables["y"][:]

geoproj = ccrs.Geostationary(
    central_longitude=gip.longitude_of_projection_origin,
    sweep_axis=gip.sweep_angle_axis,
    satellite_height=gip.perspective_point_height,
)

x *= gip.perspective_point_height
y *= gip.perspective_point_height
x = x[1510:1710]
y = y[650:850]

# x=x[1510:2310]
# y=y[50:850]

xx, yy = np.meshgrid(x, y)
goesnc.close()

goesvis = Dataset("/home/jcintineo/probsevere_oconus/src/static/GOES_East_C02.nc", "r")
xv = goesvis.variables["x"][:]
yv = goesvis.variables["y"][:]
xv *= gip.perspective_point_height
yv *= gip.perspective_point_height
xv = xv[1510 * 4 : 1710 * 4]
yv = yv[650 * 4 : 850 * 4]
xxv, yyv = np.meshgrid(xv, yv)
goesvis.close()

outdir = "/home/jcintineo/probsevere_oconus/src/segmentation/20180718_ch02051315_400x400_118dates_FED_forReport/"
utils.mkdir_p(outdir)
zf = "/apollo/grain/jcintineo/AI/OCONUS/seg/unet_train400_5frames_eni_hrrr_ghrsst.zarr/"  #'unet_train400_5frames_eni.zarr'
model = os.path.join(
    "/home",
    "jcintineo",
    "probsevere_oconus",
    "src",
    "unet",
    "118dates",
    "0.0003",
    "c02051315_nf16_fed_WEIRD",
    "model-09-0.102128.h5",
)
conv_model = load_model(model, compile=False)

ds = zarr.open(zf, "r")

# preddictfile = os.path.join('/home','jcintineo','probsevere_oconus','src','unet','new_800x800','c13fed_5dates_5frames','preds_labs.pkl')
# preddict = pickle.load(open(preddictfile,'rb'))

# preds = preddict['conv_preds'].reshape(288,800,800)


dates = ds.ABI_date[:]
times = ds.ABI_time[:]
ind = np.where((dates == 20180718) & (times >= 1457) & (times <= 1532))

dates = dates[ind]
times = times[ind]
sorted_ind = np.argsort(times)
ind = ind[0]

# get aspect ratio for IR data
default_imgsize = 10  # inches
asp = len(y) / len(x)
if asp > 1:
    fig_width_in = default_imgsize * asp ** (-1)
    fig_height_in = default_imgsize
else:
    fig_width_in = default_imgsize
    fig_height_in = default_imgsize * asp

for i in range(len(sorted_ind)):
    print(i)
    abi_date = dates[sorted_ind[i]]
    abi_time = times[sorted_ind[i]]
    timestamp = str(abi_date) + "-" + str(abi_time).zfill(4)
    print(timestamp)

    ch02 = ds.CH02.get_orthogonal_selection(
        ([ind[sorted_ind[i]]], -1, slice(None), slice(None), slice(None))
    )
    ch02_norm = (ch02 - ds.CH02.attrs["mean"]) / ds.CH02.attrs["std"]
    ch05 = ds.CH05.get_orthogonal_selection(
        ([ind[sorted_ind[i]]], -1, slice(None), slice(None), slice(None))
    )
    ch05_norm = (ch05 - ds.CH05.attrs["mean"]) / ds.CH05.attrs["std"]
    # ch08 = ds.CH08.get_orthogonal_selection(([ind[sorted_ind[i]]],-1,slice(None),slice(None),slice(None)))
    # ch08_norm = (ch08 - ds.CH08.attrs['mean']) / ds.CH08.attrs['std']
    # ch10 = ds.CH10.get_orthogonal_selection(([ind[sorted_ind[i]]],-1,slice(None),slice(None),slice(None)))
    # ch10_norm = (ch10 - ds.CH10.attrs['mean']) / ds.CH10.attrs['std']
    # ch11 = ds.CH11.get_orthogonal_selection(([ind[sorted_ind[i]]],-1,slice(None),slice(None),slice(None)))
    # ch11_norm = (ch11 - ds.CH11.attrs['mean']) / ds.CH11.attrs['std']
    ch13 = ds.CH13.get_orthogonal_selection(
        ([ind[sorted_ind[i]]], -1, slice(None), slice(None), slice(None))
    )
    ch13_norm = (ch13 - ds.CH13.attrs["mean"]) / ds.CH13.attrs["std"]
    ch15 = ds.CH15.get_orthogonal_selection(
        ([ind[sorted_ind[i]]], -1, slice(None), slice(None), slice(None))
    )
    ch15_norm = (ch15 - ds.CH15.attrs["mean"]) / ds.CH15.attrs["std"]
    # ch16 = ds.CH16.get_orthogonal_selection(([ind[sorted_ind[i]]],-1,slice(None),slice(None),slice(None)))
    # ch16_norm = (ch16 - ds.CH16.attrs['mean']) / ds.CH16.attrs['std']

    ch02 = ch02[0, 800:1600, 0:800, 0]
    ch13 = ch13[0, ..., 0]

    # truth = ds.MAXFED_5to60.get_orthogonal_selection(([ind[sorted_ind[i]]],slice(None),slice(None),0))
    FED = ds.FED.get_orthogonal_selection(
        ([ind[sorted_ind[i]]], -1, slice(None), slice(None), slice(None))
    )
    # FED_norm = (FED - ds.FED.attrs['mean']) / ds.FED.attrs['std']
    FED = FED[0, 200:400, 0:200, 0]

    # ENI = ds.ENI.get_orthogonal_selection(([ind[sorted_ind[i]]],-1,slice(None),slice(None),slice(None)))
    # ENI = ENI[0,...,0]

    # pred_in = [ch02_norm,np.concatenate((ch08_norm,ch10_norm,ch11_norm,ch13_norm,ch15_norm,ch16_norm),axis=-1)]
    pred_in = [
        ch02_norm,
        ch05_norm,
        np.concatenate((ch13_norm, ch15_norm), axis=-1),
    ]  # np.concatenate((ch08_norm,ch10_norm,ch13_norm),axis=-1)]

    t0 = time.time()
    new_preds = conv_model.predict(pred_in)
    print("max new_preds:", np.max(new_preds))
    print(time.time() - t0)
    new_preds = new_preds[0]
    new_preds = new_preds[200:400, 0:200]
    fig = plt.figure(figsize=(fig_width_in, fig_height_in))

    ax = fig.add_axes([0, 0, 1, 1], projection=geoproj)
    # set extent of image
    xmin = np.min(x)
    xmax = np.max(x)
    ymin = np.min(y)
    ymax = np.max(y)
    ax.set_extent([xmin, xmax, ymin, ymax], crs=geoproj)
    ax.add_feature(cfeature.BORDERS)
    ax.add_feature(state_borders, edgecolor="tan")

    # irimg = ax.pcolormesh(xx,yy,ch13, vmin=180, vmax=310, cmap=plt.get_cmap('Greys'))
    # cbaxes = fig.add_axes([0.01,-0.05,0.27,0.03]) #left bottom width height
    # cbar = plt.colorbar(irimg, orientation='horizontal',pad=0.01, extendfrac=0, drawedges=0, ax=ax, ticks=np.arange(180,301,40),cax=cbaxes)
    # cbar.set_label('ABI 10.35-$\mu$m brightness temperature [K]',fontsize=10)

    visimg = ax.pcolormesh(
        xxv, yyv, np.sqrt(ch02), vmin=0, vmax=1, cmap=plt.get_cmap("Greys_r")
    )
    cbaxes = fig.add_axes([0.01, -0.05, 0.27, 0.03])  # left bottom width height
    cbar = plt.colorbar(
        visimg,
        orientation="horizontal",
        pad=0.01,
        extendfrac=0,
        drawedges=0,
        ax=ax,
        ticks=np.arange(0, 1.01, 0.2),
        cax=cbaxes,
    )
    cbar.set_label("ABI 0.64-$\mu$m reflectance []", fontsize=10)

    try:
        ENI
    except NameError:
        glmimg = ax.pcolormesh(
            xxv,
            yyv,
            ma.masked_less(zoom(FED, 4, order=0), 0.1),
            vmin=0.1,
            vmax=80,
            cmap=plt.get_cmap("autumn_r"),
            linewidth=0,
            antialiased=True,
            alpha=0.3,
        )
        cbaxes2 = fig.add_axes([0.31, -0.05, 0.27, 0.03])  # left bottom width height
        cbar2 = plt.colorbar(
            glmimg,
            orientation="horizontal",
            pad=0.01,
            extendfrac=0,
            drawedges=0,
            ax=ax,
            ticks=np.arange(10, 71, 20),
            cax=cbaxes2,
        )
        cbar2.set_label("Max GLM FED [fl (5 min)$^{-1}$]", fontsize=10)
        cbar2.set_alpha(1.0)
        cbar2.draw_all()
    else:
        eniimg = ax.pcolormesh(
            xx,
            yy,
            ma.masked_less(ENI, 0.0001),
            vmin=0.0001,
            vmax=1.0,
            cmap=plt.get_cmap("inferno"),
        )  # ,linewidth=0, antialiased=True, alpha=0.3)
        cbaxes2 = fig.add_axes([0.31, -0.05, 0.27, 0.03])  # left bottom width height
        cbar2 = plt.colorbar(
            eniimg,
            orientation="horizontal",
            pad=0.01,
            extendfrac=0,
            drawedges=0,
            ax=ax,
            ticks=np.arange(0, 1.01, 0.2),
            cax=cbaxes2,
        )
        cbar2.set_label(
            "Max ENI flash density [fl (min)$^{-1}$ (km)$^{-2}$]", fontsize=10
        )
        # cbar2.set_alpha(1.0)
        # cbar2.draw_all()

    colors = ["#0000FF", "#00FFFF", "#00FF00", "#CC00FF"]
    values = [0.1, 0.25, 0.5, 0.75]
    linewidths = [1, 1, 1, 1.5]
    labels = [str(int(v * 100)) + "%" for v in values]
    contour_img = ax.contour(
        xxv,
        yyv,
        zoom(new_preds, 4, order=0),
        values,
        colors=colors,
        linewidths=linewidths,
    )  # , cmap=plt.get_cmap('cividis'))
    custom_lines = [
        Line2D([0], [0], color=c, lw=lw * 2) for c, lw in zip(colors, linewidths)
    ]
    leg2 = ax.legend(
        custom_lines,
        labels,
        fontsize=11,
        loc="lower right",
        handlelength=1.5,
        ncol=len(labels),
        bbox_to_anchor=(1, -0.06),
    )
    ctext = plt.text(
        0.65,
        -0.09,
        "Probability of lightning in 60 min",
        fontsize=14,
        transform=ax.transAxes,
    )

    ax.set_title((" ").join(["GOES-East", "CONUS", timestamp, "UTC"]), fontsize=16)

    plt.savefig(
        os.path.join(
            outdir, ("_").join(["GOES16", "CONUS", "PLTG60", timestamp + ".png"])
        ),
        bbox_inches="tight",
    )

    # sys.exit()
