import sys, os
import numpy as np
import collections
from lightningcast import utils
import pickle


def torch_config():
    binarize = -1
    nframes = 1
    dilation = 1
    shuffle = True
    nworkers = 8  # number of workers for fit_generator for train_set

    minmax = []

    all_scaling_vals = {
        "CH02": {"mean": 0.08713, "std": 0.15314},
        "CH05": {"mean": 0.047332, "std": 0.0834376},
        "CH06": {"mean": 0.04413, "std": 0.080274},
        "CH08": {"mean": 234.73875, "std": 6.4855},
        "CH10": {"mean": 252.71758, "std": 11.06057},
        "CH11": {"mean": 274.36105, "std": 21.69784},
        "CH13": {"mean": 270.93, "std": 25.9126},
        "CH15": {"mean": 273.168, "std": 19.9895},
        "CH16": {"mean": 260.5381, "std": 17.2468},
        "FED": {"mean": 0.3536, "std": 2.341},
        "sinday": {"mean": 0.0, "std": 1.0},
        "cosday": {"mean": 0.0, "std": 1.0},
        "satzen": {"mean": 42.68, "std": 13.55424},
        "solzen": {"mean": 80.176812, "std": 38.9773},
        "lon": {"mean": -88.4005, "std": 18.07278},
        "lat": {"mean": 30.9988, "std": 10.34049},
    }

    # NB: the HRRR fields are 64x64, but at a different domain than ABI. Thus, they are not technically 1km res.
    hs = {
        "4km": ["GHRSST"],
        "2km": [
            "CH06",
            "CH07",
            "CH08",
            "CH09",
            "CH10",
            "CH11",
            "CH12",
            "CH13",
            "CH14",
            "CH15",
            "CH16",
            "FED",
            "AFA",
            "FCD",
            "MFA",
            "TOE",
            "BTD1315",
            "BTD1113",
            "u925",
            "v925",
            "u850",
            "v850",
            "u700",
            "v700",
            "u500",
            "v500",
            "mlcape",
        ],
        "1km": [
            "CH05",
            "mlcape_arr",
            "SST_arr",
            "u925_arr",
            "v925_arr",
            "u850_arr",
            "v850_arr",
            "u700_arr",
            "v700_arr",
            "u500_arr",
            "v500_arr",
        ],
        "05km": ["CH02"],
    }

    cnn = "UNet2D"
    # data = '/apollo/grain/jcintineo/AI/OCONUS/seg/unet_train400_5frames_eni_hrrr_ghrsst.zarr'
    # data = '/apollo/grain/jcintineo/AI/OCONUS/seg/netcdf400x400_eni_hrrr_ghrsst/' #assumes netcdfs are in <datadir>/<date>/
    data = []  # set empty if using train/val lists

    val_dates = [
        20180515,
        20180516,
        20180517,
        20180518,
        20180519,
        20180615,
        20180616,
        20180617,
        20180618,
        20180619,
        20180715,
        20180716,
        20180717,
        20180718,
        20180719,
        20180815,
        20180816,
        20180817,
        20180818,
        20180819,
    ]
    # val_dates = [20180601]
    train_dates = list(range(20180501, 20180515)) + list(range(20180520, 20180532))
    train_dates += list(range(20180601, 20180615))
    train_dates += list(range(20180620, 20180631))
    train_dates += list(range(20180701, 20180715))
    train_dates += list(range(20180720, 20180732))
    train_dates += list(range(20180801, 20180815))
    train_dates += list(range(20180820, 20180832))
    # train_dates = [20180701]

    # if train_dates = val_dates = [], then we are using samplelists.
    train_list = "/home/jcintineo/lightningcast/src/samplelists/training/train_0.5%_every2.pkl"  # samplelist_2019_frac0.1_2kmGLM.pkl' #train_curated.pkl'
    val_list = "/home/jcintineo/lightningcast/src/samplelists/validation/val_0.5%_every2.pkl"  # samplelist_2019_24dates_frac0.1_2kmGLM.pkl' #val_curated.pkl'
    # train_list = val_list = []
    train_dates = val_dates = []

    outdir = "pt/0.5%/c02051315_0003/"

    inputs = []

    # scalar_vars = ['satzen','cosday','sinday']
    scalar_vars = []
    if cnn.startswith("UNet"):
        ngpu = 4
        batchsize = ngpu * 1
        inputs.append(["CH02"])
        inputs.append(["CH05"])
        inputs.append(["CH13", "CH15"])  # "CH13", "CH15"
        if "400" in data:
            ps_2km = (400, 400)
        else:
            ps_2km = (736, 832)  # lustre patch size
        ps_4km = tuple([_ // 2 for _ in ps_2km])
        ps_1km = tuple([_ * 2 for _ in ps_2km])
        ps_05km = tuple([_ * 4 for _ in ps_2km])
        input_tuples = []
        for i, inp in enumerate(inputs):  # channels first for pytorch
            if inp[0] in hs["2km"]:
                input_tuples.append((len(inp), ps_2km[0], ps_2km[1]))
            elif inp[0] in hs["1km"]:
                input_tuples.append((len(inp), ps_1km[1], ps_1km[1]))
            elif inp[0] in hs["05km"]:
                input_tuples.append((len(inp), ps_05km[0], ps_05km[1]))
            elif inp[0] in hs["4km"]:
                input_tuples.append((len(inp), ps_4km[0], ps_4km[1]))
            else:
                print("Can't find " + inp + " in hs lists!")
                sys.exit()
        # make the scaling values dict from all_scaling_vals "library"
        scaling_vals = {}
        for inp in inputs:
            for ch in inp:
                scaling_vals[ch] = all_scaling_vals[ch]

        loss_fcn = "CrossEntropyLoss"  # BCEWithLogitsLoss CrossEntropyLoss 'FocalTverskyLoss' #'BCE'
        nclass = 2 if (loss_fcn == "CrossEntropyLoss") else 1

        datagen = "DatasetLUSTRE"  # DatasetNETCDF DatasetLUSTRE DatasetZARR_UNET
        binarize = 0.1
        learning_rate = 0.0003  # can perhaps do higher LR if using batchnorm
        target = (
            "FED_accum_60min_2km"  # MAXFED_5to60 FED_accum_60min FED_accum_60min_2km
        )

        class_weights = False  # {0:0.51666592, 1:15.50067251} #computed with binarization threshold = 0.99 and target = 'MAXFED_5to60'
        # {0:0.5400779 , 1:6.73785139} #computed with 400x400 binarization threshold=0.99 and target='MAXFED_5to60'
        num_conv_filters = 16
        dilation = 1
        bias_init = None  # np.array([-2.52378297]) #this np.log([pos/neg]) ; see https://www.tensorflow.org/tutorials/structured_data/imbalanced_data
        # np.array([-2.52378297]) is for 400x400 zarr file, MAXFED_5to60, binarized with FED=0.1

    channels = []
    for inp in inputs:
        channels += inp

    if scalar_vars:
        # assuming input_tuples[-1] is 2km data. This makes the 'scalars' 8km resolution.
        scalar_reduce_res_fac = srrf = 4
        scalar_tuple = (
            len(scalar_vars),
            input_tuples[-1][1] // srrf,
            input_tuples[-1][2] // srrf,
        )
        for sv in scalar_vars:
            scaling_vals[sv] = all_scaling_vals[sv]
        # input_tuples.append(scalar_tuple)
        # input_tuples.append((len(scalar_vars),))
    else:
        scalar_reduce_res_fac = 4
        scalar_tuple = ()

    return {
        "class_weights": class_weights,
        "batch_norm": False,
        "bias_init": bias_init,
        "shuffle": shuffle,
        "nframes": nframes,
        "scalar_vars": scalar_vars,
        "scalar_tuple": scalar_tuple,
        "ps_2km": ps_2km,
        "ps_1km": ps_1km,
        "ps_05km": ps_05km,
        "channels": channels,
        "l2_param": 0.0,
        "dropout_rate": 0.0,
        "batchsize": batchsize,
        "num_conv_filters": num_conv_filters,
        "filter_width": 3,
        "conv_activation": "relu",
        "learning_rate": learning_rate,
        "nepoch": 200,
        "nworkers": nworkers,
        "target_time": 60,  # in min
        "es_patience": 4,
        "rlr_factor": 0.1,
        "rlr_patience": 1,
        "rlr_cooldown": 0,
        "cnn": cnn,
        "input_tuples": input_tuples,
        "inputs": inputs,
        "val_dates": val_dates,
        "train_dates": train_dates,
        "train_list": train_list,
        "val_list": val_list,
        "target": target,
        "data": data,
        "scaling_vals": scaling_vals,
        "binarize": binarize,
        "minmax": minmax,
        "scalar_reduce_res_fac": scalar_reduce_res_fac,
        "datagen": datagen,
        "dilation": dilation,
        "ngpu": ngpu,
        "loss_fcn": loss_fcn,
        "optimizer": "AdamW",
        "outdir": outdir,
        "nclass": nclass,
    }


if __name__ == "__main__":
    outdir = sys.argv[1]
    utils.mkdir_p(outdir)
    print(torch_config())
    sys.exit()
    pickle.dump(torch_config(), open(os.path.join(outdir, "model_config.pkl"), "wb"))
