from __future__ import absolute_import, division, print_function, unicode_literals

import matplotlib

matplotlib.use("Agg")
import os

# os.environ["CUDA_DEVICE_ORDER"]="PCI_BUS_ID";
# os.environ["CUDA_VISIBLE_DEVICES"]="1";

from netCDF4 import Dataset
import numpy as np
import pandas as pd
import xarray as xr
import zarr
import matplotlib.pyplot as plt
import matplotlib.patches as mpatches
from lightningcast import performance_diagrams
from lightningcast import roc_curves
from lightningcast import attributes_diagrams
import pickle
from glob import glob
from sklearn.preprocessing import StandardScaler
import tensorflow as tf

tf.keras.backend.clear_session()
from tensorflow import keras

from sklearn.preprocessing import StandardScaler
from tensorflow.keras.models import Model, save_model, load_model
from tensorflow.keras.layers import (
    Dense,
    Activation,
    Conv3D,
    Conv2D,
    Input,
    AveragePooling2D,
    MaxPooling2D,
    Flatten,
    LeakyReLU,
    Dropout,
)
from tensorflow.keras.layers import SpatialDropout2D, BatchNormalization
from tensorflow.keras.optimizers import SGD, Adam
from tensorflow.keras.callbacks import EarlyStopping, ModelCheckpoint, ReduceLROnPlateau
from tensorflow.keras.regularizers import l2
import tensorflow.keras.backend as K
from scipy.ndimage import gaussian_filter
from sklearn.utils import class_weight
from sklearn.metrics import mean_squared_error, roc_auc_score
from sklearn.metrics import auc as skl_auc
from sklearn.model_selection import train_test_split, StratifiedKFold
import sys
from lightningcast import utils
from datetime import datetime, timedelta
import time
import argparse

pltg = os.environ["PLTG"]
sys.path.insert(0, os.path.join(pltg, "src", "static/"))
from ABIcmaps import irw_ctc, wv_cimss_r
import logging
from logging.config import dictConfig
from lightningcast.logging_def import *
from lightningcast import keras_metrics
from lightningcast.LRFinder import LRFinder

outdir = "unet/2dates/0.0003/c020513_nf32_400x400_2GPU_inmemory"
utils.mkdir_p(outdir)

zf = "/apollo/grain/jcintineo/AI/OCONUS/seg/unet_train400_5frames_eni_hrrr_ghrsst.zarr"  # unet_CONUS_800x800.zarr'

ds = zarr.open(zf, "r")

train_idx = slice(0, 300)
val_idx = slice(300, 600)


# training
inputs = []

chan = "CH02"
mean = ds[chan].attrs["mean"]
std = ds[chan].attrs["std"]
data = (
    ds[chan].get_orthogonal_selection((train_idx, [-1], slice(None), slice(None), [-1]))
    - mean
) / std
data = np.squeeze(data, axis=1)
inputs.append(data)

chan = "CH05"
mean = ds[chan].attrs["mean"]
std = ds[chan].attrs["std"]
data = (
    ds[chan].get_orthogonal_selection((train_idx, [-1], slice(None), slice(None), [-1]))
    - mean
) / std
data = np.squeeze(data, axis=1)
inputs.append(data)

chan = "CH13"
mean = ds[chan].attrs["mean"]
std = ds[chan].attrs["std"]
ch13 = (
    ds[chan].get_orthogonal_selection((train_idx, [-1], slice(None), slice(None), [-1]))
    - mean
) / std
ch13 = np.squeeze(ch13, axis=1)

# chan='CH15'
# mean = ds[chan].attrs['mean']
# std = ds[chan].attrs['std']
# ch15 = (ds[chan].get_orthogonal_selection((train_idx,slice(None),slice(None),[-1])) - mean) / std

# chan='CH16'
# mean = ds[chan].attrs['mean']
# std = ds[chan].attrs['std']
# ch16 = (ds[chan].get_orthogonal_selection((train_idx,slice(None),slice(None),[-1])) - mean) / std
# inputs.append(np.concatenate((ch13,ch15,ch16),axis=-1))
inputs.append(ch13)

# validation
val_inputs = []

chan = "CH02"
mean = ds[chan].attrs["mean"]
std = ds[chan].attrs["std"]
data = (
    ds[chan].get_orthogonal_selection((val_idx, [-1], slice(None), slice(None), [-1]))
    - mean
) / std
data = np.squeeze(data, axis=1)
val_inputs.append(data)

chan = "CH05"
mean = ds[chan].attrs["mean"]
std = ds[chan].attrs["std"]
data = (
    ds[chan].get_orthogonal_selection((val_idx, [-1], slice(None), slice(None), [-1]))
    - mean
) / std
data = np.squeeze(data, axis=1)
val_inputs.append(data)

chan = "CH13"
mean = ds[chan].attrs["mean"]
std = ds[chan].attrs["std"]
ch13 = (
    ds[chan].get_orthogonal_selection((val_idx, [-1], slice(None), slice(None), [-1]))
    - mean
) / std
ch13 = np.squeeze(ch13, axis=1)

# chan='CH15'
# mean = ds[chan].attrs['mean']
# std = ds[chan].attrs['std']
# ch15 = (ds[chan].get_orthogonal_selection((val_idx,slice(None),slice(None),[-1])) - mean) / std

# chan='CH16'
# mean = ds[chan].attrs['mean']
# std = ds[chan].attrs['std']
# ch16 = (ds[chan].get_orthogonal_selection((val_idx,slice(None),slice(None),[-1])) - mean) / std
# val_inputs.append(np.concatenate((ch13,ch15,ch16),axis=-1))
val_inputs.append(ch13)

early_stopping = EarlyStopping(monitor="val_loss", patience=5)
mcp_save = ModelCheckpoint(
    os.path.join(outdir, "model-{epoch:02d}-{val_loss:03f}.h5"),
    save_best_only=True,
    monitor="val_loss",
    mode="min",
)
reduce_lr_loss = ReduceLROnPlateau(
    monitor="val_loss", cooldown=2, verbose=1, factor=0.1, patience=2, mode="min"
)
callbacks = [early_stopping, mcp_save, reduce_lr_loss]


# train model
from lightningcast import tf_models

tf_model = getattr(tf_models, "unet_2D")

print(inputs[0].shape, inputs[1].shape, inputs[2].shape)
print(val_inputs[0].shape, val_inputs[1].shape, val_inputs[2].shape)

from lightningcast import tf_config

config = tf_config.tf_config()
print(config["input_tuples"])

if config["ngpu"] > 1:
    # Create a MirroredStrategy.
    strategy = tf.distribute.MirroredStrategy()
    print("Number of devices: {}".format(strategy.num_replicas_in_sync))
    with strategy.scope():
        conv_model = tf_model(config["input_tuples"], options=config)
else:
    conv_model = tf_model(config["input_tuples"], options=config)

train_out = ds["MAXFED_5to60"].get_orthogonal_selection(
    (train_idx, slice(None), slice(None), [-1])
)
train_out = np.squeeze(train_out, axis=-1)
print(train_out.shape)
print("training...")
history = conv_model.fit(
    inputs,
    train_out,
    validation_data=val_inputs,
    batch_size=1,
    epochs=3,
    shuffle=True,
    callbacks=callbacks,
)
