import numpy as np
import tensorflow as tf
import matplotlib.pyplot as plt
import os, sys

import inspect

current_dir = os.path.dirname(os.path.abspath(inspect.getfile(inspect.currentframe())))
sys.path.insert(0, os.path.dirname(current_dir))

from lightningcast import utils

os.environ["CUDA_DEVICE_ORDER"] = "PCI_BUS_ID"
os.environ["CUDA_VISIBLE_DEVICES"] = "-1"


chans = ["CH02", "CH05", "CH13"]


def print_variable_names(tfrecord_filename):
    # Create a TFRecordDataset to read the TFRecord file(s)
    dataset = tf.data.TFRecordDataset(tfrecord_filename)

    # Iterate through the records in the dataset
    for record in dataset:
        # Parse the record into a TensorFlow Example
        example = tf.train.Example.FromString(record.numpy())

        # Extract the feature names from the Example
        feature_names = example.features.feature.keys()

        # Print the feature names
        print("Variable Names:", list(feature_names))


def parse_tfrecord_fn(example):
    feature_description = {
        "CH02": tf.io.FixedLenFeature([], tf.string),
        "CH05": tf.io.FixedLenFeature([], tf.string),
        "CH08": tf.io.FixedLenFeature([], tf.string),
        "CH09": tf.io.FixedLenFeature([], tf.string),
        "CH10": tf.io.FixedLenFeature([], tf.string),
        "CH11": tf.io.FixedLenFeature([], tf.string),
        "CH13": tf.io.FixedLenFeature([], tf.string),
        "CH15": tf.io.FixedLenFeature([], tf.string),
        "FED": tf.io.FixedLenFeature([], tf.string),
        "FED_accum_60min_2km": tf.io.FixedLenFeature([], tf.string),
        "REF10": tf.io.FixedLenFeature([], tf.string)
        #    "FED_accum_15min_2km": tf.io.FixedLenFeature([], tf.string),
        #    "SFC_ELEV": tf.io.FixedLenFeature([], tf.string),
    }
    features = tf.io.parse_single_example(example, feature_description)

    # example["image"] = tf.io.decode_jpeg(example["FED_accum_15min_2km"], channels=1)
    example = {}
    image = tf.io.parse_tensor(features["REF10"], tf.uint8)
    image = tf.reshape(image, [960, 960, 1])
    example["REF10"] = image
    image = tf.io.parse_tensor(features["CH02"], tf.uint8)
    image = tf.reshape(image, [480 * 4, 480 * 4, 1])
    example["CH02"] = image
    image = tf.io.parse_tensor(features["CH13"], tf.uint8)
    image = tf.reshape(image, [480, 480, 1])
    example["CH13"] = image
    image = tf.io.parse_tensor(features["FED_accum_60min_2km"], tf.uint8)
    image = tf.reshape(image, [480, 480, 1])
    example["target"] = image
    # example["image"] = np.random.random((128,128)) #tf.io.decode_jpeg(example["image"], channels=3)
    return example


# outfile = '/ships22/grain/probsevere/LC/tfrecs3/goes16/2020/20200601/goes16_20200601-184100_Y530_X50.tfrec'
outfile = "/ships22/grain/probsevere/LC/tfrecs3/goes16/2019/20190601/goes16_20190601-235100_Y530_X50.tfrec"

print_variable_names(outfile)

raw_dataset = tf.data.TFRecordDataset(outfile)
parsed_dataset = raw_dataset.map(parse_tfrecord_fn)

# fig,axes = plt.subplots(nrows=2,ncols=2,figsize=(8,8))


for features in parsed_dataset.take(1):
    # for key in features.keys():
    #    if key != "image":
    #        print(f"{key}: {features[key]}")

    fig, axes = plt.subplots(nrows=2, ncols=2, figsize=(10, 10))
    for ii, ax in enumerate(axes.ravel()):
        ax.axis("off")
        if ii == 0:
            ax.imshow(features["CH02"].numpy(), interpolation="none")
        elif ii == 1:
            ax.imshow(features["CH13"].numpy(), interpolation="none")
        elif ii == 2:
            ref10 = features["REF10"].numpy()
            ref10[ref10 == 0] = -999
            ax.imshow(
                utils.unbytescale(ref10, 0, 75), vmin=0, vmax=40, interpolation="none"
            )  # 147 corresponds to 40 dBZ
        else:
            ax.imshow(features["target"].numpy(), interpolation="none")
    # plt.imshow(features.numpy(), interpolation='none')
    plt.show()
    sys.exit()

    #  fig,axes = plt.subplots(nrows=2,ncols=2,figsize=(8,8))
    for ii, ax in enumerate(axes.ravel()):
        ax.imshow(features[chans[ii]].numpy())
        ax.axis("off")
        ax.set_title(chans[ii])

    # axes[0].imshow(features['CH02'].numpy())
    # axes[0].off()
    # axes[0].set_title('CH02')

    # ax1.imshow(features['CH05'].numpy())
    # ax1.off()
    # ax1.set_title('CH05')

    # ax1.imshow(features['CH13'].numpy())
    # ax1.off()
    # ax1.set_title('CH13')

    plt.show()
